//adc_read_main.c

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdbool.h>
#include <signal.h>
#include <unistd.h>

#include <pigpio.h>

#include "control_adc.h"
#include "control_common.h"

#define ADC_CHANNELS (8)

//シグナルハンドラ関数(sigint)
static void sigint_handler(int signum);

int main(void)
{
    //sigaction構造体用変数
    struct sigaction act, oldact;
    int count = 0;
    unsigned int adc_result[ADC_CHANNELS];

    //pigpio初期化
    if(gpioInitialise() < 0){
        printf("pigpio initialize error. \n");
        exit(-1);
    }

    //Ctrl+Cを押した時にシグナルハンドラを実行する
	act.sa_handler = sigint_handler;
	//SIGINT 受信時に動作する
	sigaction(SIGINT, &act, &oldact);

    adc_init();


    while(1){
        count ++;
        sleep_ms(100); //1ループあたり100msのサイクルとする

        for(int ch = 0; ch < ADC_CHANNELS; ch ++){
            adc_result[ch] = adc_get_value(ch);
        }

        printf("%05d , %4d, %4d, %4d, %4d, %4d, %4d, %4d, %4d \n",
            count,  
            adc_result[0], adc_result[1], adc_result[2], adc_result[3],
            adc_result[4], adc_result[5], adc_result[6], adc_result[7]
            );
        
        
        

    }

}


//シグナルハンドラ関数(sigint)
void sigint_handler(int signum)
{
    terminate_adc();
    gpioTerminate();  
    exit(0);
}

