//adc_read_motor_speed_main.c.c

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdbool.h>
#include <signal.h>
#include <unistd.h>

#include <pigpio.h>

#include "control_adc.h"
#include "control_motor.h"
#include "control_common.h"

#define ADC_VR0 (4)
#define ADC_VR1 (5)
#define ADC_VR2 (6)

//シグナルハンドラ関数(sigint)
static void sigint_handler(int signum);

int main(void)
{
    //sigaction構造体用変数
    struct sigaction act, oldact;
    int count = 0;
    unsigned int adc_VR0, adc_VR1;
    int duty_left, duty_right;

    //pigpio初期化
    if(gpioInitialise() < 0){
        printf("pigpio initialize error. \n");
        exit(-1);
    }



    //Ctrl+Cを押した時にシグナルハンドラを実行する
	act.sa_handler = sigint_handler;
	//SIGINT 受信時に動作する
	sigaction(SIGINT, &act, &oldact);

    adc_init();
    motor_init();


    while(1){
        count ++;
        sleep_ms(100); //1ループあたり100msのサイクルとする

        adc_VR0 = adc_get_value(ADC_VR0);
        adc_VR1 = adc_get_value(ADC_VR1);

        duty_left = ((((double)(ADC_MAX - adc_VR0) / ADC_MAX) * 100.0)  - 50) * 2 ;
        duty_right = ((((double)(ADC_MAX - adc_VR1) / ADC_MAX) * 100.0)  - 50) * 2;
        
        motor_set_speed_left(duty_left);
        motor_set_speed_right(duty_right);
        
        printf("%5d, duty_left: %4d %%, duty_right: %4d %%\n",count,
            duty_left, duty_right);
        

    }

}


//シグナルハンドラ関数(sigint)
void sigint_handler(int signum)
{
    terminate_adc();
    gpioTerminate();  
    exit(0);
}

