//control_motor.c

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdbool.h>
#include <signal.h>
#include <unistd.h>

#include <pigpio.h>

//モーターへつながる端子のGPIO番号
#define PWM_MOT_L1 (12)
#define PWM_MOT_L2 (18)
#define PWM_MOT_R1 (13)
#define PWM_MOT_R2 (19)

//ON幅を指定する数値の最大値
#define ON_WIDTH_MAX (250)

//DUTY比のステップ値
#define ON_WIDTH_STEP (5)

//PWM制御するときの周波数
#define PWM_FREQ (1000)

//DUTY比は0〜100で指定されるが、実際にON幅としてレジスタに設定する値はことなる
//DUTY比とON幅設定値の比
#define DUTY_ON_WIDTH_RATIO (ON_WIDTH_MAX/100.0)


//モーター関連を初期化
void motor_init(void)
{
    //モーターに接続されているGPIOの設定
    //各端子を出力に設定する
    gpioSetMode(PWM_MOT_L1, PI_OUTPUT);
    gpioSetMode(PWM_MOT_L2, PI_OUTPUT);
    gpioSetMode(PWM_MOT_R1, PI_OUTPUT);
    gpioSetMode(PWM_MOT_R2, PI_OUTPUT);

    //各端子のPWM周波数を設定する
    gpioSetPWMfrequency(PWM_MOT_L1, PWM_FREQ);
    gpioSetPWMfrequency(PWM_MOT_L2, PWM_FREQ);
    gpioSetPWMfrequency(PWM_MOT_R1, PWM_FREQ);
    gpioSetPWMfrequency(PWM_MOT_R2, PWM_FREQ);

}


//左モーターの回転速度と方向を指定: 0: 停止, 100: 前進全開, -100: 後退全開
void motor_set_speed_left(int speed){
    int duty;
    //100越え、-100以下は100, -100にする
    if(speed > 100) {
        speed = 100;
    }
    if(speed < -100){
        speed = -100;
    }

    duty = speed * DUTY_ON_WIDTH_RATIO;
    if(duty == 0){
        //停止
        gpioPWM(PWM_MOT_L1, 0);
        gpioPWM(PWM_MOT_L2, 0);

    } else if(duty > 0){
        //正転
        gpioPWM(PWM_MOT_L1, 0);
        gpioPWM(PWM_MOT_L2, duty);
    }else if(duty < 0){
        //逆転
        gpioPWM(PWM_MOT_L1, -1 * duty);
        gpioPWM(PWM_MOT_L2, 0);
    } else {
        //do nothing
    }
}

//右モーターの回転速度と方向を指定: 0: 停止, 100: 前進全開, -100: 後退全開
void motor_set_speed_right(int speed){
    int duty;
    
    //100越え、-100以下は100, -100にする
    if(speed > 100) {
        speed = 100;
    }
    if(speed < -100){
        speed = -100;
    }
    
    duty = speed * DUTY_ON_WIDTH_RATIO;
    if(duty == 0){
        //停止
        gpioPWM(PWM_MOT_R1, 0);
        gpioPWM(PWM_MOT_R2, 0);

    } else if(duty > 0){
        //正転
        gpioPWM(PWM_MOT_R1, duty);
        gpioPWM(PWM_MOT_R2, 0);
    }else if(duty < 0){
        //逆転
        gpioPWM(PWM_MOT_R1, 0);
        gpioPWM(PWM_MOT_R2, -1 * duty);
    } else {
        //do nothing
    }
}
