//blink_led2.c

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdbool.h>
#include <signal.h>
#include <unistd.h>

#include <pigpio.h>

//指定された時間(ms)休止する
void sleep_ms(unsigned int ms);

//シグナルハンドラ関数(sigint)
static void sigint_handler(int signum);

//LEDが接続されているGPIOのGPIO番号
#define GPIO_LED_ST0 4
#define GPIO_LED_ST1 5
#define GPIO_LED_ST2 6

int main(void)
{
    //sigaction構造体用変数
    struct sigaction act, oldact;
    int count = 0;
    //pigpio初期化
    if(gpioInitialise() < 0){
        printf("pigpio initialize error. \n");
        exit(-1);
    }

    //LEDが接続されているGPIOの初期化
    gpioSetMode(GPIO_LED_ST0, PI_OUTPUT);
    gpioSetMode(GPIO_LED_ST1, PI_OUTPUT);
    gpioSetMode(GPIO_LED_ST2, PI_OUTPUT);

    //Ctrl+Cを押した時にシグナルハンドラを実行する
	act.sa_handler = sigint_handler; // シグナルハンドラの関数ポインタ（先頭アドレス）を設定する
	sigaction(SIGINT, &act, &oldact); // シグナル名とそれに対応するシグナルハンドラ関数をシステムに登録する

    while(1){
        count ++;
        sleep_ms(10);  //10ms待つ、1ループあたり10msのサイクルとする

        //カウンタによって時間を計る
        if(count < 50){ //count : 0 ~ 49
            gpioWrite(GPIO_LED_ST0, 0); /*Low:点灯, Hi: 消灯*/
            gpioWrite(GPIO_LED_ST1, 0);
            gpioWrite(GPIO_LED_ST2, 0);
        } else if((count < 100)){// count: 50~99
            gpioWrite(GPIO_LED_ST0, 1); /*Low:点灯, Hi: 消灯*/
            gpioWrite(GPIO_LED_ST1, 1);
            gpioWrite(GPIO_LED_ST2, 1);
        } else {// count: 100~
            //clear count
            count = 0;
        }
    }
}

//指定された時間(ms)休止する
//休止している間、別のスレッドまたはタスクが実行可能
void sleep_ms(unsigned int ms) {
    struct timespec req;
    req.tv_sec = 0;              // 秒
    req.tv_nsec = ms * 1000*1000;  // ms(ミリ秒) =  ms * 1000(us) * 1000(ns)
    nanosleep(&req, NULL);
}

//シグナルハンドラ関数(sigint)
void sigint_handler(int signum)
{
    gpioWrite(GPIO_LED_ST0, 1); /*LEDを消灯(Hi: Off))*/
    gpioWrite(GPIO_LED_ST1, 1);
    gpioWrite(GPIO_LED_ST2, 1);
    gpioTerminate();    //pigpioモジュールを終了する
    exit(0);    //プログラム終了
}

