//motor_control01.c

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdbool.h>
#include <signal.h>
#include <unistd.h>

#include <pigpio.h>

//指定された時間(ms)休止する
void sleep_ms(unsigned int ms);

//シグナルハンドラ関数(sigint)
static void sigint_handler(int signum);


//モーターへつながる端子のGPIO番号
#define PWM_MOT_L1 (12)
#define PWM_MOT_L2 (18)
#define PWM_MOT_R1 (13)
#define PWM_MOT_R2 (19)

//ON幅を指定する数値の最大値
#define ON_WIDTH_MAX (250)

//ON幅のステップ値
#define ON_WIDTH_STEP (5)

//PWM制御するときの周波数
#define PWM_FREQ (1000)

#define ONE_CYCLE_MS (50)

int main(void)
{
    //sigaction構造体用変数
    struct sigaction act, oldact;
    int count = 0;
    //pigpio初期化
    if(gpioInitialise() < 0){
        printf("pigpio initialize error. \n");
        exit(-1);
    }

    //モーターに接続されているGPIOの設定
    //各端子を出力に設定する
    gpioSetMode(PWM_MOT_L1, PI_OUTPUT);
    gpioSetMode(PWM_MOT_L2, PI_OUTPUT);
    gpioSetMode(PWM_MOT_R1, PI_OUTPUT);
    gpioSetMode(PWM_MOT_R2, PI_OUTPUT);

    //各端子のPWM周波数を設定する
    gpioSetPWMfrequency(PWM_MOT_L1, PWM_FREQ);
    gpioSetPWMfrequency(PWM_MOT_L2, PWM_FREQ);
    gpioSetPWMfrequency(PWM_MOT_R1, PWM_FREQ);
    gpioSetPWMfrequency(PWM_MOT_R2, PWM_FREQ);

    //Ctrl+Cを押した時にシグナルハンドラを実行する
	act.sa_handler = sigint_handler;
	//SIGINT 受信時に動作する
	sigaction(SIGINT, &act, &oldact);

    while(1){
        for(int on_width = 0; on_width < ON_WIDTH_MAX; on_width += ON_WIDTH_STEP){
            //正転
            gpioPWM(PWM_MOT_L1, 0);
            gpioPWM(PWM_MOT_L2, on_width);

            printf("LEFT MOTOR CW Duty: %5.1f \n", 100 * on_width / (double)ON_WIDTH_MAX);

            sleep_ms(ONE_CYCLE_MS);
        }
        for(int on_width = ON_WIDTH_MAX; on_width > 0; on_width -= ON_WIDTH_STEP){
            //正転
            gpioPWM(PWM_MOT_L1, 0);
            gpioPWM(PWM_MOT_L2, on_width);

            printf("LEFT MOTOR CW Duty: %5.1f \n", 100 * on_width / (double)ON_WIDTH_MAX);

            sleep_ms(ONE_CYCLE_MS);
        }
        for(int on_width = 0; on_width < ON_WIDTH_MAX; on_width += ON_WIDTH_STEP){
            //正転
            gpioPWM(PWM_MOT_R1, 0);
            gpioPWM(PWM_MOT_R2, on_width);

            printf("RIGHT MOTOR CW Duty: %5.1f \n", 100 * on_width / (double)ON_WIDTH_MAX);

            sleep_ms(ONE_CYCLE_MS);
        }
        for(int on_width = ON_WIDTH_MAX; on_width > 0; on_width -= ON_WIDTH_STEP){
            //正転
            gpioPWM(PWM_MOT_R1, 0);
            gpioPWM(PWM_MOT_R2, on_width);

            printf("RIGHT MOTOR CW Duty: %5.1f \n", 100 * on_width / (double)ON_WIDTH_MAX);

            sleep_ms(ONE_CYCLE_MS);
        }
    }


}

//指定された時間(ms)休止する
//休止している間、別のスレッドまたはタスクが実行可能
void sleep_ms(unsigned int ms) {
    struct timespec req;
    req.tv_sec = 0;              // 秒
    req.tv_nsec = ms * 1000*1000;  // ms(ミリ秒) =  ms * 1000(us) * 1000(ns)
    nanosleep(&req, NULL);
}


//シグナルハンドラ関数(sigint)
void sigint_handler(int signum)
{
    gpioPWM(PWM_MOT_L1, 0);
    gpioPWM(PWM_MOT_L2, 0);
    gpioPWM(PWM_MOT_R1, 0);
    gpioPWM(PWM_MOT_R2, 0);
    gpioTerminate();  
    exit(0);
}

