//read_sw.c

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdbool.h>
#include <signal.h>
#include <unistd.h>

#include <pigpio.h>

//指定された時間(ms)休止する
void sleep_ms(unsigned int ms);

//シグナルハンドラ関数(sigint)
static void sigint_handler(int signum);

//SWが接続されているGPIOのGPIO番号
#define GPIO_SW_P0 (16)
#define GPIO_SW_P1 (17)
#define GPIO_SW_P2 (20)

int main(void)
{
    //sigaction構造体用変数
    struct sigaction act, oldact;
    int count = 0;
    int sw_p0, sw_p1, sw_p2;

    //pigpio初期化
    if(gpioInitialise() < 0){
        printf("pigpio initialize error. \n");
        exit(-1);
    }

    //SWが接続されているGPIOの初期化
    gpioSetMode(GPIO_SW_P0, PI_INPUT);
    gpioSetMode(GPIO_SW_P1, PI_INPUT);
    gpioSetMode(GPIO_SW_P2, PI_INPUT);

    //Ctrl+Cを押した時にシグナルハンドラを実行する
	act.sa_handler = sigint_handler;
	//SIGINT 受信時に動作する
	sigaction(SIGINT, &act, &oldact);


    while(1){
        count ++;
        sleep_ms(100); //100mn待つ、1ループあたり100msのサイクルとする
        sw_p0 = gpioRead(GPIO_SW_P0); //スイッチを読み込む
        sw_p1 = gpioRead(GPIO_SW_P1);
        sw_p2 = gpioRead(GPIO_SW_P2);
        
        //スイッチの状態を表示する
        printf("%04d, %1d, %1d, %1d \n", count, sw_p0, sw_p1, sw_p2);

    }

}

//指定された時間(ms)休止する
//休止している間、別のスレッドまたはタスクが実行可能
void sleep_ms(unsigned int ms) {
    struct timespec req;
    req.tv_sec = 0;              // 秒
    req.tv_nsec = ms * 1000*1000;  // ms(ミリ秒) =  ms * 1000(us) * 1000(ns)
    nanosleep(&req, NULL);
}


//シグナルハンドラ関数(sigint)
void sigint_handler(int signum)
{
    gpioTerminate();  //pigpioモジュールを終了する
    exit(0);
}

