//blink_led2.c

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdbool.h>
#include <signal.h>
#include <unistd.h>

#include <pigpio.h>

//指定された時間(ms)休止する
void sleep_ms(unsigned int ms);

//シグナルハンドラ関数(sigint)
static void sigint_handler(int signum);

//LEDが接続されているGPIOのGPIO番号
#define GPIO_LED_ST0 (4)
#define GPIO_LED_ST1 (5)
#define GPIO_LED_ST2 (6)

//SWが接続されているGPIOのGPIO番号
#define GPIO_SW_P0 (16)
#define GPIO_SW_P1 (17)
#define GPIO_SW_P2 (20)


int main(void)
{
    //sigaction構造体用変数
    struct sigaction act, oldact;
    int count = 0;
    int sw_p0, sw_p1, sw_p2;

    //pigpio初期化
    if(gpioInitialise() < 0){
        printf("pigpio initialize error. \n");
        exit(-1);
    }

    //LEDが接続されているGPIOの初期化
    gpioSetMode(GPIO_SW_P0, PI_INPUT);
    gpioSetMode(GPIO_SW_P1, PI_INPUT);
    gpioSetMode(GPIO_SW_P2, PI_INPUT);
    //SWが接続されているGPIOの初期化
    gpioSetMode(GPIO_LED_ST0, PI_OUTPUT);
    gpioSetMode(GPIO_LED_ST1, PI_OUTPUT);
    gpioSetMode(GPIO_LED_ST2, PI_OUTPUT);

    //Ctrl+Cを押した時にシグナルハンドラを実行する
	act.sa_handler = sigint_handler;
	//SIGINT 受信時に動作する
	sigaction(SIGINT, &act, &oldact);


    while(1){
        count ++;
        sleep_ms(100); //100ms待つ、1サイクルあたり100msのサイクルとする
        sw_p0 = gpioRead(GPIO_SW_P0);
        sw_p1 = gpioRead(GPIO_SW_P1);
        sw_p2 = gpioRead(GPIO_SW_P2);

        if(sw_p0 == 1){
            gpioWrite(GPIO_LED_ST0, 0); /*Low:点灯, Hi: 消灯*/
        } else {
            gpioWrite(GPIO_LED_ST0, 1);
        }
       
        if(sw_p1 == 1){
            gpioWrite(GPIO_LED_ST1, 0); /*Low:点灯, Hi: 消灯*/
        } else {
            gpioWrite(GPIO_LED_ST1, 1);
        }

        if(sw_p2 == 1){
            gpioWrite(GPIO_LED_ST2, 0); /*Low:点灯, Hi: 消灯*/
        } else {
            gpioWrite(GPIO_LED_ST2, 1);
        }

        printf("%04d, %1d, %1d, %1d \n", count, sw_p0, sw_p1, sw_p2);
    }

}

//指定された時間(ms)休止する
//休止している間、別のスレッドまたはタスクが実行可能
void sleep_ms(unsigned int ms) {
    struct timespec req;
    req.tv_sec = 0;              // 秒
    req.tv_nsec = ms * 1000*1000;  // ms(ミリ秒) =  ms * 1000(us) * 1000(ns)
    nanosleep(&req, NULL);
}



//シグナルハンドラ関数(sigint)
void sigint_handler(int signum)
{
    gpioTerminate();    //pigpioを初期化する
    exit(0);
}

