//sound_buzzer.c

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdbool.h>
#include <signal.h>
#include <unistd.h>

#include <pigpio.h>

//指定された時間(ms)休止する
void sleep_ms(unsigned int ms);

//シグナルハンドラ関数(sigint)
static void sigint_handler(int signum);

//ブザーが接続されているGPIO番号
#define GPIO_BUZZER 21 

int main(void)
{
    //sigaction構造体用変数
    struct sigaction act, oldact;
    int count = 0;
    //pigpio初期化
    if(gpioInitialise() < 0){
        printf("pigpio initialize error. \n");
        exit(-1);
    }

    //buzzerが接続されているGPIOの初期化
    gpioSetMode(GPIO_BUZZER, PI_OUTPUT);
    //PWM周波数を設定
    gpioSetPWMfrequency(GPIO_BUZZER, 440);

    //Ctrl+Cを押した時にシグナルハンドラを実行する
	act.sa_handler = sigint_handler;
	//SIGINT 受信時に動作する
	sigaction(SIGINT, &act, &oldact);


    while(1){
        count ++;
        sleep_ms(10); //10ms待つ、1サイクルあたり10msとする
        
        //ブザーを吹鳴・停止するタイミングをつくる
        if(count < 50){ //count : 0 ~ 49
            //BUZZER吹鳴、DUTY比を50%にする(設定値0: 0%, 設定値255: 100%)
            gpioPWM(GPIO_BUZZER, 127);
        } else if((count < 100)){// count: 50~99
            //BUZZER停止、DUTY比を0%にすれば停止する
            gpioPWM(GPIO_BUZZER, 0);
        } else {// count: 100~
            //clear count
            count = 0;
        }


    }

}

//指定された時間(ms)休止する
//休止している間、別のスレッドまたはタスクが実行可能
void sleep_ms(unsigned int ms) {
    struct timespec req;
    req.tv_sec = 0;              // 秒
    req.tv_nsec = ms * 1000*1000;  // ms(ミリ秒) =  ms * 1000(us) * 1000(ns)
    nanosleep(&req, NULL);
}


//シグナルハンドラ関数(sigint)
void sigint_handler(int signum)
{
    gpioPWM(GPIO_BUZZER, 0);
    gpioTerminate();  
    exit(0);
}

