//control_sw.c

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdbool.h>
#include <signal.h>
#include <unistd.h>

#include <pigpio.h>

#include "control_sw.h"

//SWが接続されているGPIOのGPIO番号
#define GPIO_SW_P0 (16)
#define GPIO_SW_P1 (17)
#define GPIO_SW_P2 (20)

#define NUM_OF_SW (3) //スイッチが何個あるか
#define SW_CHATTERING_COUNT_MAX (4)//同一レベル何サイクルで【確定】するか

//チャタリング処理のための変数
static unsigned int sw_hi_count[NUM_OF_SW];     //Hi入力が続いているサイクル数
static unsigned int sw_low_count[NUM_OF_SW];    //Low入力が続いているサイクル数
static e_sw_state sw_state[NUM_OF_SW];

//このモジュールを初期化する
void sw_init(void){
    int i;

    for(int i = 0; i < NUM_OF_SW; i++){
        sw_hi_count[i] = 0;
        sw_low_count[i] = SW_CHATTERING_COUNT_MAX + 1;
        sw_state[i] = SW_OFF;
    }
    //SWが接続されているGPIOの初期化
    gpioSetMode(GPIO_SW_P0, PI_INPUT);
    gpioSetMode(GPIO_SW_P1, PI_INPUT);
    gpioSetMode(GPIO_SW_P2, PI_INPUT);

    printf("SW initialized \n");
}

//SWの状態をチェックする。10ms毎に呼び出すこと。
void sw_check(void){

    //SW_P0のチャタリングを処理する
    if(gpioRead(GPIO_SW_P0) == 0){  //入力がLowのとき
        sw_low_count[SW_P0] ++; //Lowのカウントをインクリメント
        sw_hi_count[SW_P0] = 0; //Hiのカウントはクリア
    } else if (gpioRead(GPIO_SW_P0) == 1){ //入力がHiのとき
        sw_low_count[SW_P0] = 0;  //Lowのカウントはクリア
        sw_hi_count[SW_P0]  ++;   //Hiのカウントをインクリメント

    } else {
        //do nothing
    }
    //SW_P1のチャタリングを処理する
    if(gpioRead(GPIO_SW_P1) == 0){
        sw_low_count[SW_P1] ++;
        sw_hi_count[SW_P1] = 0;
    } else if (gpioRead(GPIO_SW_P1) == 1){
        sw_low_count[SW_P1] = 0;
        sw_hi_count[SW_P1]  ++;

    } else {
        //do nothing
    }

    //SW_P2のチャタリングを処理する
    if(gpioRead(GPIO_SW_P2) == 0){
        sw_low_count[2] ++;
        sw_hi_count[2] = 0;
    } else if (gpioRead(GPIO_SW_P2) == 1){
        sw_low_count[2] = 0;
        sw_hi_count[2]  ++;

    } else {
        //do nothing
    }

    //各カウンタの値によって、OFF⇒ON切り替わり、ON確定、ON⇒OFF切り替わり、OFF確定をチェックする
    for(int sw = 0; sw < NUM_OF_SW; sw++){
        
        if(sw_low_count[sw] == SW_CHATTERING_COUNT_MAX){
            sw_state[sw] = SW_TOGGLED_OFF; //ON⇒OFF切り替わり
        } else if(sw_low_count[sw] > SW_CHATTERING_COUNT_MAX){
            sw_state[sw] = SW_OFF;  //OFF確定
            sw_low_count[sw] = SW_CHATTERING_COUNT_MAX + 1;
        }

        if(sw_hi_count[sw] == SW_CHATTERING_COUNT_MAX){
            sw_state[sw] = SW_TOGGLED_ON; //OFF⇒ON切り替わり
        } else if(sw_hi_count[sw] > SW_CHATTERING_COUNT_MAX){
            sw_state[sw] = SW_ON;  //ON確定
            sw_hi_count[sw] = SW_CHATTERING_COUNT_MAX + 1;
        } else {
            //do nothing
        }
    }

}

//スイッチの状態を取得して返す
e_sw_state sw_get_state(e_sw_num sw_num){
    return(sw_state[sw_num]);
}

