//toggle_sw_led_main.c 

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdbool.h>
#include <signal.h>
#include <unistd.h>

#include <pigpio.h>

#include "control_sw.h"

//指定された時間(ms)休止する
void sleep_ms(unsigned int ms);

//シグナルハンドラ関数(sigint)
static void sigint_handler(int signum);

//GPIOで点灯するLEDとGPIO番号の対応
#define GPIO_LED_ST0 (4)
#define GPIO_LED_ST1 (5)
#define GPIO_LED_ST2 (6)

int main(void)
{
    //sigaction構造体用変数
    struct sigaction act, oldact;
    int count = 0;
    int led_st0 = 1, led_st1 = 1, led_st2 = 1;

    //pigpio初期化
    if(gpioInitialise() < 0){
        printf("pigpio initialize error. \n");
        exit(-1);
    }

    //RaspberryPiのGPIOに接続されているLEDの初期化

    gpioSetMode(GPIO_LED_ST0, PI_OUTPUT);
    gpioSetMode(GPIO_LED_ST1, PI_OUTPUT);
    gpioSetMode(GPIO_LED_ST2, PI_OUTPUT);

    gpioWrite(GPIO_LED_ST0, 1);
    gpioWrite(GPIO_LED_ST1, 1);
    gpioWrite(GPIO_LED_ST2, 1); 

    //Ctrl+Cを押した時にシグナルハンドラを実行する
	act.sa_handler = sigint_handler;
	//SIGINT 受信時に動作する
	sigaction(SIGINT, &act, &oldact);

    sw_init();


    while(1){
        count ++;
        sleep_ms(10); //wait 10ms
        
        sw_check();

        if(sw_get_state(SW_P0) == SW_TOGGLED_ON){
            if(led_st0 == 0) {
                led_st0 = 1;
            } else {
                led_st0 = 0;
            }
            gpioWrite(GPIO_LED_ST0, led_st0);
            printf("%04d, %1d, %1d, %1d \n", count, led_st0, led_st1, led_st2);
        }
        if(sw_get_state(SW_P1) == SW_TOGGLED_ON){
            if(led_st1 == 0) {
                led_st1 = 1;
            } else {
                led_st1 = 0;
            }
            gpioWrite(GPIO_LED_ST1, led_st1);
            printf("%04d, %1d, %1d, %1d \n", count, led_st0, led_st1, led_st2);
        }
        if(sw_get_state(SW_P2) == SW_TOGGLED_ON){
            if(led_st2 == 0) {
                led_st2 = 1;
            } else {
                led_st2 = 0;
            }
            gpioWrite(GPIO_LED_ST2, led_st2);
            printf("%04d, %1d, %1d, %1d \n", count, led_st0, led_st1, led_st2);
        }                
    }
}

//指定された時間(ms)休止する
//休止している間、別のスレッドまたはタスクが実行可能
void sleep_ms(unsigned int ms) {
    struct timespec req;
    req.tv_sec = 0;              // 秒
    req.tv_nsec = ms * 1000*1000;  // ms(ミリ秒) =  ms * 1000(us) * 1000(ns)
    nanosleep(&req, NULL);
}


//シグナルハンドラ関数(sigint)
void sigint_handler(int signum)
{
    gpioTerminate();  
    exit(0);
}

