//udp_recv_motor_main.c
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <signal.h>


#include <pigpio.h>

#include "control_common.h"
#include "control_motor.h"


//sigaction構造体用変数
struct sigaction	act, oldact;

int sock_desc_sig; //シグナルハンドラでcloseするためのソケットディスクプタ

char time_now_str[TIME_STR_MAX]; //

#define UDP_PORT 12345  //使用するポート番号(クライアント側と合わせておく)
#define BUF_SIZE 1024


//シグナルハンドラ関数
//ctrl+cが押されたときに、ソケットを閉じて終了する
void sigint_handler(int signum)
{
	printf(" [SIGINTを受信しました。socketを閉じて終了します]\n");
    close(sock_desc_sig);
    motor_set_speed_left(0);
    motor_set_speed_right(0);
    
    gpioTerminate();  	exit(0);	//プログラム終了
}


int main(void) {
    int sd;
    struct sockaddr_in serveraddr, clientaddr;
    char udp_payload[BUF_SIZE];
    socklen_t clientlen = sizeof(clientaddr);
    int duty_left, duty_right;

    //pigpio初期化
    if(gpioInitialise() < 0){
        printf("pigpio initialize error. \n");
        exit(-1);
    }


    //ユーザ定義ハンドラ関数の指定
	act.sa_handler = sigint_handler;
	//SIGINT 受信時に動作する
	sigaction(SIGINT, &act, &oldact);


    // ソケットを作成
    sd = socket(AF_INET, SOCK_DGRAM, 0);
    sock_desc_sig = sd;  //シグナルハンドラで終了させるためのsoket descripterを保存
    if (sd < 0) {
        perror("socket creation failed");
        exit(EXIT_FAILURE);
    }

    memset(&serveraddr, 0, sizeof(serveraddr));
    memset(&clientaddr, 0, sizeof(clientaddr));

    // サーバの情報を設定
    serveraddr.sin_family = AF_INET;
    serveraddr.sin_addr.s_addr = INADDR_ANY;
    serveraddr.sin_port = htons(UDP_PORT);

    // ソケットをアドレスにバインド
    if (bind(sd, (const struct sockaddr *)&serveraddr, sizeof(serveraddr)) < 0) {
        perror("bind failed");
        exit(EXIT_FAILURE);
    }

    printf("UDP プロトコルを使って、次のポート番号でメッセージを待ち受けます: %d\n", UDP_PORT);

    while(1) {
        //クライアントからのメッセージを待ち受ける
        int recv_size = recvfrom(sd, (char *)udp_payload, BUF_SIZE, 0, (struct sockaddr *)&clientaddr, &clientlen);
        udp_payload[recv_size] = '\0'; // 終端文字をセット

        get_time_now(time_now_str);
        printf("%s : %s\n", time_now_str,udp_payload);

        sscanf(udp_payload + sizeof("REMOTE"), "%d, %d",
            &duty_left,
            &duty_right
 
        );

        motor_set_speed_left(duty_left);
        motor_set_speed_right(duty_right);

    }

    close(sd);
    return 0;
}
