//control_udp_recv.c
#include <stdio.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pthread.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdbool.h>
#include <fcntl.h>

#include "control_udp_recv.h"
#include "control_common.h"



//mutex変数を初期化
static pthread_mutex_t mutex_udp_recv = PTHREAD_MUTEX_INITIALIZER;

static bool udp_recv_thread_terminate_request;   //スレッド終了要求フラグ
static bool udp_recv_thread_terminated;          //スレッド終了完了フラグ

int sock_desc_sig; //シグナルハンドラでcloseするためのソケットディスクプタ

bool new_data_arrved;
char udp_payload[UDP_PAYLOAD_SIZE];

//スレッド関数のプロトタイプ宣言
static void* thread_func_udp_recv(void* arg);

static void udp_recv_set_payload(char *payload);


//このモジュールを初期化する
void udp_recv_init(void){
    pthread_t thread_udp_recv_id;

    new_data_arrved = false;

    printf("udp_recv_thread_starting.\n");

    // スレッド生成
    if (pthread_create(&thread_udp_recv_id, NULL, thread_func_udp_recv, NULL) != 0) {
        perror("pthread_create");
        exit(-1);
    }
    //スレッドにデタッチ属性を設定する
    pthread_detach(thread_udp_recv_id);

}

//スレッドを終了する
void terminate_udp_recv(void){
    udp_recv_thread_terminated = false;       //スレッド終了済みフラグをOFFにする
    udp_recv_thread_terminate_request = true; //スレッド終了要求フラグをONにする

    printf("udp_recv thread terminating.\n");
    close(sock_desc_sig);

    while(1){
        sleep_ms(10);
        if(udp_recv_thread_terminated == true){
            break;
        }
    }


    printf("udp_recv thread terminated.\n");
}

//スレッド関数
static void* thread_func_udp_recv(void* arg) {
    int sd;
    struct sockaddr_in serveraddr, clientaddr;
    char udp_payload_buffer[UDP_PAYLOAD_SIZE];
    socklen_t clientlen = sizeof(clientaddr);

   // ソケットを作成
    sd = socket(AF_INET, SOCK_DGRAM, 0);
    sock_desc_sig = sd;  //シグナルハンドラで終了させるためのsoket descripterを保存


    if (sd < 0) {
        perror("socket creation failed");
        exit(EXIT_FAILURE);
    }

    memset(&serveraddr, 0, sizeof(serveraddr));
    memset(&clientaddr, 0, sizeof(clientaddr));

    // サーバの情報を設定
    serveraddr.sin_family = AF_INET;
    serveraddr.sin_addr.s_addr = INADDR_ANY;
    serveraddr.sin_port = htons(UDP_PORT);

    // ソケットをアドレスにバインド
    if (bind(sd, (const struct sockaddr *)&serveraddr, sizeof(serveraddr)) < 0) {
        perror("bind failed");
        exit(EXIT_FAILURE);
    }

    printf("UDP プロトコルを使って、次のポート番号でメッセージを待ち受けます: %d\n", UDP_PORT);


    while (1) {
        //スレッド終了要求があれば、スレッドを終了する(無限ループを抜ける)
        if(udp_recv_thread_terminate_request == true){
            break;
        }

        //クライアントからのメッセージを待ち受ける
        int recv_size = recvfrom(sd, (char *)udp_payload_buffer, UDP_PAYLOAD_SIZE, 0, (struct sockaddr *)&clientaddr, &clientlen);
        udp_payload_buffer[recv_size] = '\0'; // 終端文字をセット

        udp_recv_set_payload(udp_payload_buffer);

    }

    udp_recv_thread_terminated = true;
    return NULL; // 終了
}

//受信したUDPペイロードのデータをセットする
static void udp_recv_set_payload(char *payload){
    pthread_mutex_lock(&mutex_udp_recv);
    new_data_arrved = true;
    strncpy(udp_payload, payload, UDP_PAYLOAD_SIZE );
    pthread_mutex_unlock(&mutex_udp_recv);

}

//新たに受信したパケットのペイロード・データを取得する
void udp_recv_get_payload(char * payload){
    pthread_mutex_lock(&mutex_udp_recv);

    strncpy(payload, udp_payload, UDP_PAYLOAD_SIZE);
    new_data_arrved = false;
    pthread_mutex_unlock(&mutex_udp_recv);

}

//前回ペイロード・データを取得して以降に、あらたなデータを受信したか
bool udp_recv_is_new_data_arrived(void){
    bool result;
    
    pthread_mutex_lock(&mutex_udp_recv);

    result = new_data_arrved;
    pthread_mutex_unlock(&mutex_udp_recv);

    return(result);
}