//udp_recv_motor_thread_main.c
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <signal.h>

#include <stdbool.h>


#include <pigpio.h>

#include "control_common.h"
#include "control_udp_recv.h"
#include "control_motor.h"


//sigaction構造体用変数
struct sigaction	act, oldact;


char time_now_str[TIME_STR_MAX]; //

//シグナルハンドラ関数
//ctrl+cが押されたときに、ソケットを閉じて終了する
void sigint_handler(int signum)
{
	printf(" [SIGINTを受信しました。]\n");
    terminate_udp_recv();
    motor_set_speed_left(0);
    motor_set_speed_right(0);
    
    gpioTerminate();  	exit(0);	//プログラム終了
}


int main(void) {

    unsigned int count = 0;
    int duty_left = 0, duty_right = 0;
    char udp_recv_data[UDP_PAYLOAD_SIZE] = "";
    bool udp_data_arrived = false;

    unsigned int last_udp_arrived_count = 0;

    //pigpio初期化
    if(gpioInitialise() < 0){
        printf("pigpio initialize error. \n");
        exit(-1);
    }

    udp_recv_init();


    //ユーザ定義ハンドラ関数の指定
	act.sa_handler = sigint_handler;
	//SIGINT 受信時に動作する
	sigaction(SIGINT, &act, &oldact);


 
    while(1) {
        sleep_ms(100);  //1ループあたり100ms
        //新たなパケットが到着したか
        udp_data_arrived = udp_recv_is_new_data_arrived();
        if(udp_data_arrived == true){ //新たなパケットが到着していれば
            //到着しているパケットを取得する
            udp_recv_get_payload(udp_recv_data);
            if(strncmp(udp_recv_data, "REMOTE", sizeof("REMOTE") - 1) == 0){
                sscanf(udp_recv_data + sizeof("REMOTE"), "%d, %d",
                    &duty_left,
                    &duty_right
        
                );
                last_udp_arrived_count = count;
            }
        } else{ //パケットが到着していなければ
            //パケットが連続して到着していない時間をカウントし、
            //カウント値が一定以上であれば、モーターを停止する
            if( (last_udp_arrived_count - count) > 5 ){
                duty_left = 0;
                duty_right = 0;

                memset(udp_recv_data,'\0',UDP_PAYLOAD_SIZE);
            }
        }
        get_time_now(time_now_str);
        printf("%s : %d, %s\n", time_now_str, udp_data_arrived,udp_recv_data);

        motor_set_speed_left(duty_left);
        motor_set_speed_right(duty_right);

        count ++;
    }

    return 0;
}
