//control_udp_recv.c
#include <stdio.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pthread.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdbool.h>
#include <fcntl.h>

#include "control_udp_recv.h"
#include "control_common.h"



//mutex変数を初期化
static pthread_mutex_t mutex_udp_recv = PTHREAD_MUTEX_INITIALIZER;

static bool udp_recv_thread_terminate_request;   //スレッド終了要求フラグ
static bool udp_recv_thread_terminated;          //スレッド終了完了フラグ

int sock_desc_sig; //シグナルハンドラでcloseするためのソケットディスクプタ

bool new_data_arrved;
char udp_payload[UDP_PAYLOAD_SIZE];

//スレッド関数
static void* thread_func_udp_recv(void* arg);

static void udp_recv_set_payload(char *payload);


//このモジュールを初期化する
void udp_recv_init(void){
    pthread_t thread_udp_recv_id;

    new_data_arrved = false;

    printf("udp_recv_thread_starting.\n");

    // スレッド生成
    if (pthread_create(&thread_udp_recv_id, NULL, thread_func_udp_recv, NULL) != 0) {
        perror("pthread_create");
        exit(-1);
    }
    //スレッドにデタッチ属性を設定する
    pthread_detach(thread_udp_recv_id);

}

//スレッドを終了する
void terminate_udp_recv(void){
    udp_recv_thread_terminated = false;       //スレッド終了済みフラグをOFFにする
    udp_recv_thread_terminate_request = true; //スレッド終了要求フラグをONにする

    printf("udp_recv thread terminating.\n");
    close(sock_desc_sig);

    while(1){
        sleep_ms(10);
        if(udp_recv_thread_terminated == true){
            break;
        }
    }


    printf("udp_recv thread terminated.\n");
}


//スレッド関数
static void *thread_func_udp_recv(void *arg) {
    int sockfd; //ソケットディスクリプタ
    struct sockaddr_in servaddr, cliaddr;
    char udp_payload_buffer[UDP_PAYLOAD_SIZE];  //受信データを保存しておくバッファ
    socklen_t len;
    fd_set readfds; //ファイルディスクリプタセット
    int recv_size;

    // ソケットを作成
    if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
        perror("socket creation failed");
        exit(EXIT_FAILURE);
    }

    memset(&servaddr, 0, sizeof(servaddr));
    memset(&cliaddr, 0, sizeof(cliaddr));

    // サーバアドレスの設定
    servaddr.sin_family = AF_INET;
    servaddr.sin_addr.s_addr = INADDR_ANY;
    servaddr.sin_port = htons(UDP_PORT);

    // ソケットにアドレスをバインド
    if (bind(sockfd, (const struct sockaddr *)&servaddr, sizeof(servaddr)) < 0) {
        perror("bind failed");
        exit(EXIT_FAILURE);
    }

    // ソケットをノンブロッキングモードに設定
    //通常の方法だとパケットを受信するまでブロックするが、
    //この方法はパケットが来なくてもブロックせずに次の命令を実行できる
    int flags = fcntl(sockfd, F_GETFL, 0);
    fcntl(sockfd, F_SETFL, flags | O_NONBLOCK);

    // select()のタイムアウト設定（10ms）
    //タイムアウト時間が経過するまでパケットが来なかったら、次の命令が実行さえっる
    struct timeval tv;
    tv.tv_sec = 0;
    tv.tv_usec = 10000; //10,000 us => 10ms

    // パケットを受信する
    while (1) {
        //スレッド終了要求があれば、スレッドを終了する(無限ループを抜ける)
        if(udp_recv_thread_terminate_request == true){
            break;
        }
        //使用するソケットディスクリプタをクリア
        FD_ZERO(&readfds);
        //使用するソケットディスクリプタ(sockfd)を設定
        FD_SET(sockfd, &readfds);

        //パケットが到着しているかをチェック
        int ret = select(sockfd + 1, &readfds, NULL, NULL, &tv);
        if (ret == -1) {
            perror("select error");
            exit(EXIT_FAILURE);
        } else if (ret == 0) {
            // タイムアウト、パケットは到着していない
            continue;
        }

        // 指定したソケットディスクリプタが利用可能かチェック
        if (FD_ISSET(sockfd, &readfds)) {
            //パケットが到着、sockfcが利用可能であれば、受信して取り込む
            len = sizeof(cliaddr);
            recv_size = recvfrom(sockfd, udp_payload_buffer, UDP_PAYLOAD_SIZE, 0, (struct sockaddr *)&cliaddr, &len);
            if (recv_size < 0) {
                perror("recvfrom error");
                continue;
            }
            udp_payload_buffer[recv_size] = '\0'; // ヌル終端を確実にする
            udp_recv_set_payload(udp_payload_buffer);

        }
    }
    udp_recv_thread_terminated = true;
    close(sockfd);
    return NULL;
}
//////






static void udp_recv_set_payload(char *payload){
    pthread_mutex_lock(&mutex_udp_recv);
    new_data_arrved = true;
    strncpy(udp_payload, payload, UDP_PAYLOAD_SIZE );
    pthread_mutex_unlock(&mutex_udp_recv);

}

void udp_recv_get_payload(char * payload){
    pthread_mutex_lock(&mutex_udp_recv);

    strncpy(payload, udp_payload, UDP_PAYLOAD_SIZE);
    new_data_arrved = false;
    pthread_mutex_unlock(&mutex_udp_recv);

}

bool udp_recv_is_new_data_arrived(void){
    bool result;
    
    pthread_mutex_lock(&mutex_udp_recv);

    result = new_data_arrved;
    pthread_mutex_unlock(&mutex_udp_recv);

    return(result);
}