//write_lcd_main.c

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdbool.h>
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>

#include <sys/ioctl.h>
#include <linux/i2c-dev.h>

#include <stdlib.h>

#include "control_common.h"
#include "control_i2c.h"

//LCDのI2Cアドレス
#define ADDR_LCD (0x3e)

//LCDパネルを操作するコマンドレジスタアドレス
#define LCD_COMMAND_INST (0x00)
#define LCD_COMMAND_CHAR (0x40)

#define LCD_LINE (2)    //LCDの行数
//LCDに表示できる文字数
#define LCD_MSG_LENGTH (17)

//LCDパネルの0行目か1行目か
typedef enum { LCD_LINE_0, LCD_LINE_1 } e_lcd_line;

//LCDを制御するコマンドのデータ
static const unsigned char command_initialize_lcd[] = {LCD_COMMAND_INST, 0x38, 0x39, 0x14, 0x70, 0x56, 0x6c};
static const unsigned char command_hide_cursor[] = {LCD_COMMAND_INST, 0x0c};
static const unsigned char command_show_cursor[] = {LCD_COMMAND_INST, 0x0d};
static const unsigned char command_clear_lcd_show_cursor[] = {LCD_COMMAND_INST, 0x38, 0x0d, 0x01};
static const unsigned char command_clear_lcd[] = {LCD_COMMAND_INST, 0x38, 0x01};

static const unsigned char command_cursor_line_1[] = {LCD_COMMAND_INST, 0x80};
static const unsigned char command_cursor_line_2[] = {LCD_COMMAND_INST, 0xc0};

//文字列をLCDパネルに表示する
static void lcd_set_string(e_lcd_line line, char *str);
//文字を１文字表示する
static void lcd_putchar(char moji);
//LCDパネルをクリアする
static void lcd_clear(void);


#define I2C_DEV_FILE "/dev/i2c-1"  // Raspberry PiのI2Cデバイスファイル

int main(void){
    int count = 0;
    //初期化コマンドを送る
    i2c_write_multi_byte(ADDR_LCD, command_initialize_lcd, sizeof(command_initialize_lcd));
    //カーソルを表示する
    i2c_write_multi_byte(ADDR_LCD, command_clear_lcd_show_cursor, sizeof(command_clear_lcd_show_cursor));


    while(1){
        count ++;
        lcd_clear();

        if( (count % 2) == 0){
            lcd_set_string(LCD_LINE_0, "hello");
        } else {
            lcd_set_string(LCD_LINE_1, "    world!!");

        }

        sleep_ms(500);
    }

    return(0);
}

//LCDに1文字表示する
static void lcd_putchar(char moji){
    i2c_write_byte(ADDR_LCD, LCD_COMMAND_CHAR, moji);
}
    

//LCDに文字列を設定し表示する
static void lcd_set_string(e_lcd_line line, char *str){
    int i;

    if(line == LCD_LINE_0){
        i2c_write_multi_byte(ADDR_LCD, command_cursor_line_1, sizeof(command_cursor_line_1));
    } else if(line == LCD_LINE_1){
        i2c_write_multi_byte(ADDR_LCD, command_cursor_line_2, sizeof(command_cursor_line_2));
    } else {
        /* do nothing */
    }

    for( i = 0; i < LCD_MSG_LENGTH; i++){
        //null文字を検出したらやめる
        if(str[i] == 0x00){
            break;
        }
        lcd_putchar(str[i]);    //LCDに1文字出力
    }
}

//LCDに表示されている文字をすべて消す
static void lcd_clear(void){
    i2c_write_multi_byte(ADDR_LCD, command_clear_lcd, sizeof(command_clear_lcd));
}
