//vehicle_control_buzzer.c
//ブザーに関連する処理をおこなうモジュール

#include <stdio.h>
#include <pthread.h>
#include <unistd.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdbool.h>

#include <pigpio.h>

#include "vehicle_control_buzzer.h"
#include "vehicle_control_common.h"

//ブザーが接続されているGPIO番号
#define GPIO_BUZZER 21 

//ブザー周波数の初期値
#define BUZZER_FREQUENCY_INIT (440)

//
#define BUZZER_CONTINUE_CYCLE (25)

//ブザー周波数
static int frequency;

// ブザー継続カウント(1カウントあたり10ms)
static volatile int32_t sustain_count = 0;


static bool buzzer_thread_terminate_request;     //スレッド終了要求フラグ
static bool buzzer_thread_terminated;  //スレッド終了完了フラグ

// スレッド関数
void* thread_func_buzzer(void* arg);


//ブザーを初期化する
void buzzer_init(void){
    pthread_t thread_buzzer_id;
    frequency = BUZZER_FREQUENCY_INIT;

    gpioSetMode(GPIO_BUZZER, PI_OUTPUT);
    gpioSetPWMfrequency(GPIO_BUZZER, frequency);

    // スレッド生成
    if (pthread_create(&thread_buzzer_id, NULL, thread_func_buzzer, NULL) != 0) {
        perror("pthread_create");
        exit(-1);
    }
    pthread_detach(thread_buzzer_id);
}

//ブザー周波数を設定する
void buzzer_set_frequency(int freq){
    frequency = freq;
    if(frequency < 100) {
        frequency = 100;
    }

    if(frequency > 2000){
        frequency = 2000;
    }
}

// スレッド関数
void* thread_func_buzzer(void* arg) {
    while (1) {
         //スレッド終了要求があれば、スレッドを終了する(無限ループを抜ける)
        if(buzzer_thread_terminate_request == true){
            break;
        }
        //1サイクルは10ms
        sleep_10ms();
        //継続カウントをデクリメント
        sustain_count --;
        //継続時間
        if(sustain_count <= 0){
            //継続時間がゼロになったらブザー止める
            gpioPWM(GPIO_BUZZER, 0);
            sustain_count = 0;
            
        } else {
            /* do nothing*/
        }
    }

    buzzer_thread_terminated = true;
    return NULL; // 実際には終了しない
}

//スレッドを終了させる。スレッドが終了するまでブロックする
void terminate_buzzer(void){
    buzzer_thread_terminated = false;
    buzzer_thread_terminate_request = true;

    printf("buzzer thread terminating.\n");
    while(1){
        sleep_10ms(); //10msごとにスレッドが終了したかチェックする
        if(buzzer_thread_terminated == true){
            break;
        }
    }

    printf("buzzer thread terminated.\n");

}

//ブザーに信号を出力する
static void buzzer_start(void){
    //周波数を設定
    gpioSetPWMfrequency(GPIO_BUZZER, frequency);
    //BUZZER吹鳴
    gpioPWM(GPIO_BUZZER, 127);

}

//ブザーを短時間鳴らす(ピッ！)
void buzzer_on_short(void){
    sustain_count = 20;
    buzzer_start();

}

//ブザーを短時間鳴らす(ピーーッ)
void buzzer_on_long(void){
    sustain_count = 100;
    buzzer_start();
}


//ブザーを鳴らし続ける(ピーーーーー)
void buzzer_on(void){
    
    sustain_count = 10000;
    buzzer_start();


}

//ブザーを停止する
void buzzer_off(void){
    //継続時間をなくす
    sustain_count = 0;
}

//
void buzzer_continue(void){
    if(sustain_count > 0){
        sustain_count = BUZZER_CONTINUE_CYCLE;
        
    } else {
        sustain_count = BUZZER_CONTINUE_CYCLE;
        buzzer_start();

    }


}
