//vehicle_control_lcd.c
//LCD関連の処理をおこなうモジュール

#include <stdio.h>
#include <pigpio.h>
#include <fcntl.h>
#include <unistd.h>

#include <sys/ioctl.h>
#include <linux/i2c-dev.h>

#include <stdlib.h>

#include "vehicle_control_common.h"
#include "vehicle_control_lcd.h"
#include "vehicle_control_i2c.h"

//LCDのI2Cアドレス
#define ADDR_LCD (0x3e)

//LCDパネルを操作するコマンドレジスタアドレス
#define LCD_COMMAND_INST (0x00)
#define LCD_COMMAND_CHAR (0x40)

#define LCD_LINE (2)    //LCDの行数


//LCDを制御するコマンドのデータ
static const unsigned char command_initialize_lcd[] = {LCD_COMMAND_INST, 0x38, 0x39, 0x14, 0x70, 0x56, 0x6c};
static const unsigned char command_hide_cursor[] = {LCD_COMMAND_INST, 0x0c};
static const unsigned char command_show_cursor[] = {LCD_COMMAND_INST, 0x0d};
static const unsigned char command_clear_lcd_show_cursor[] = {LCD_COMMAND_INST, 0x38, 0x0d, 0x01};
static const unsigned char command_clear_lcd[] = {LCD_COMMAND_INST, 0x38, 0x01};

static const unsigned char command_cursor_line_1[] = {LCD_COMMAND_INST, 0x80};
static const unsigned char command_cursor_line_2[] = {LCD_COMMAND_INST, 0xc0};

//このモジュールを初期化する
void lcd_init(void){
    //初期化コマンドを送る
    i2c_write_multi_byte(ADDR_LCD, command_initialize_lcd, sizeof(command_initialize_lcd));
    i2c_write_multi_byte(ADDR_LCD, command_clear_lcd_show_cursor, sizeof(command_clear_lcd_show_cursor));
    i2c_write_multi_byte(ADDR_LCD, command_cursor_line_1, sizeof(command_cursor_line_1));

}

//LCDに文字列を設定し表示する
void lcd_set_string(e_lcd_line line, char *str){
    int i;

    if(line == LCD_LINE_0){
        i2c_write_multi_byte(ADDR_LCD, command_cursor_line_1, sizeof(command_cursor_line_1));
    } else if(line == LCD_LINE_1){
        i2c_write_multi_byte(ADDR_LCD, command_cursor_line_2, sizeof(command_cursor_line_2));
    } else {
        /* do nothing */
    }

    for( i = 0; i < LCD_MSG_LENGTH; i++){
        //null文字を検出したらやめる
        if(str[i] == 0x00){
            break;
        }
        lcd_putchar(str[i]);    //LCDに1文字出力
    }
}

//カーソルを表示する
void lcd_cursor_on(void){
    i2c_write_multi_byte(ADDR_LCD, command_show_cursor, sizeof(command_show_cursor));
}

//カーソルを非表示にする
void lcd_cursor_off(void){
    i2c_write_multi_byte(ADDR_LCD, command_hide_cursor, sizeof(command_hide_cursor));
}

//LCDに1文字表示する
void lcd_putchar(char moji){
    i2c_write_byte(ADDR_LCD, LCD_COMMAND_CHAR, moji);
}

//LCDに表示されている文字をすべて消す
void lcd_clear(void){
    i2c_write_multi_byte(ADDR_LCD, command_clear_lcd, sizeof(command_clear_lcd));
}
