//vehicle_control_main

#include <stdio.h>
#include "common.h"
#include "read_sw.h"
#include "vehicle_control_main.h"
#include "gpio.h"
#include "reflect_sensor.h"
#include "motor.h"
#include "adc.h"
#include "led_indicator.h"
#include "vehicle_control_main_line_tracing.h"


typedef enum {
	LINETRACING_MODE_WAITING_START,
	LINETRACING_MODE_START_BEEPING,
	LINETRACING_MODE_RUNNING,
	NUMBER_OF_LINETRACING_MODE_STATE
} e_lintraceing_mode_state;

static e_lintraceing_mode_state current_lintraceing_mode_state;
static e_lintraceing_mode_state next_lintraceing_mode_state;

static void wait_for_start_button(void);
static void start_signal_sounding(void);
static void line_tracing(void);

static void detect_drift(void);
static buzzer_drift(void);
static void determine_vehicle_direction_and_speed(void);


static SINT16 start_signal_sound_count;

static UINT8  drift_left;
static UINT8  drift_right;

static SINT16 throttle_max;

#define START_SIGNAL_SOUND_CONT_INIT (500)
#define BUZZER_INTERVAL_INIT (75)

void vehicle_control_main_line_tracing_init(void)
{
	current_lintraceing_mode_state = LINETRACING_MODE_WAITING_START;
	next_lintraceing_mode_state = LINETRACING_MODE_WAITING_START;
	start_signal_sound_count = START_SIGNAL_SOUND_CONT_INIT;
	
	drift_left = 0;
	drift_right = 0;
	
	throttle_max = 30;
	
	led_indicator_turn_on(LED_ST0);
	led_indicator_turn_off(LED_ST1);
	led_indicator_turn_off(LED_ST2);
	return;
}

void vehicle_control_main_line_tracing_slot_a(void)
{
	switch(current_lintraceing_mode_state){
	case	LINETRACING_MODE_WAITING_START:
		wait_for_start_button();
		break;
	case	LINETRACING_MODE_START_BEEPING:
		start_signal_sounding();
		break;
	case	LINETRACING_MODE_RUNNING:
		line_tracing();
		break;
	default:
		break;
	}
	
	
	
	return;
}


void vehicle_control_main_line_tracing_slot_b(void)
{

	
	
	return;
}

void vehicle_control_main_line_tracing_slot_c(void)
{


	current_lintraceing_mode_state = next_lintraceing_mode_state;
	return;
}

static void wait_for_start_button(void)
{
	e_reclect_sensor_value sensor_L0, sensor_L1, sensor_L2;
	e_reclect_sensor_value sensor_R0, sensor_R1, sensor_R2;	
	static UINT8 vol_update_cycle = 0;
	UINT16 volume;
	char msg[18];

	
	volume = adc_get_value(AD_VR0) / 40;
	if(volume > 100){
		volume = 100;	
	}
	
	vol_update_cycle ++;
	
	if(vol_update_cycle >= 10){
		vol_update_cycle = 0;
		sprintf(msg, "SPEED(VR0): %2d", volume);
		lcd_set_message(1, msg);
	}
	
	if(read_sw_state(SW_P0) == SW_STATE_PUSHED){
		next_lintraceing_mode_state = LINETRACING_MODE_START_BEEPING;
		start_signal_sound_count = START_SIGNAL_SOUND_CONT_INIT;
		throttle_max = volume; /* ϒR̒lőXbglɎw */
	}
	
	
	//


	
	sensor_L0 = reflect_sensor_get_value(RS_OUT_L0);
	sensor_L1 = reflect_sensor_get_value(RS_OUT_L1);
	sensor_L2 = reflect_sensor_get_value(RS_OUT_L2);

	sensor_R0 = reflect_sensor_get_value(RS_OUT_R0);
	sensor_R1 = reflect_sensor_get_value(RS_OUT_R1);
	sensor_R2 = reflect_sensor_get_value(RS_OUT_R2);


	
	if( sensor_L0 == RS_LINE_ON ){
		led_indicator_turn_on(LED_IND4);
	} else {
		led_indicator_turn_off(LED_IND4);
	}
	
	if( sensor_L1 == RS_LINE_ON ){
		led_indicator_turn_on(LED_IND3);
	} else {
		led_indicator_turn_off(LED_IND3);
		
	}

	if( sensor_L2 == RS_LINE_ON ){
		led_indicator_turn_on(LED_IND2);
	} else {
		led_indicator_turn_off(LED_IND2);
		
	}
	
	
	if( sensor_R0 == RS_LINE_ON ){
		led_indicator_turn_on(LED_IND5);
	} else {
		led_indicator_turn_off(LED_IND5);
	}
	
	if( sensor_R1 == RS_LINE_ON ){
		led_indicator_turn_on(LED_IND6);
	} else {
		led_indicator_turn_off(LED_IND6);
		
	}

	if( sensor_R2 == RS_LINE_ON ){
		led_indicator_turn_on(LED_IND7);
	} else {
		led_indicator_turn_off(LED_IND7);
		
	}

	
	return;
}

static void start_signal_sounding(void)
{
	start_signal_sound_count --;
	if(start_signal_sound_count == START_SIGNAL_SOUND_CONT_INIT - 1){
		buzzer_on_middle_L();
		lcd_set_message(1, "3...");

		led_indicator_turn_off(LED_IND0);
		led_indicator_turn_off(LED_IND1);
		led_indicator_turn_on(LED_IND2);
		led_indicator_turn_on(LED_IND3);
		led_indicator_turn_on(LED_IND4);
		led_indicator_turn_on(LED_IND5);
		led_indicator_turn_on(LED_IND6);
		led_indicator_turn_on(LED_IND7);

	} else if( start_signal_sound_count == ( (START_SIGNAL_SOUND_CONT_INIT / 4) * 3)){
		buzzer_on_middle_L();
		lcd_set_message(1, "2..");

		led_indicator_turn_off(LED_IND2);
		led_indicator_turn_off(LED_IND7);

	} else if( start_signal_sound_count == ( (START_SIGNAL_SOUND_CONT_INIT / 4) * 2)){
		buzzer_on_middle_L();
		lcd_set_message(1, "1.");
		
		led_indicator_turn_off(LED_IND3);
		led_indicator_turn_off(LED_IND6);
	} else if( start_signal_sound_count == ( (START_SIGNAL_SOUND_CONT_INIT / 4) * 1)){
		buzzer_on_long_H();
		next_lintraceing_mode_state = LINETRACING_MODE_RUNNING;
		lcd_set_message(1, "!!!START!!!");
		
		led_indicator_turn_off(LED_ST0);
		led_indicator_turn_on(LED_ST1);
		
		led_indicator_turn_on(LED_IND0);
		led_indicator_turn_on(LED_IND1);

		led_indicator_turn_off(LED_IND4);
		led_indicator_turn_off(LED_IND5);
} else if( start_signal_sound_count == 0){
		//buzzer_on_short_L();
		
	}
}

static void line_tracing(void)
{
	
	detect_drift();	/* Ê肷 */
	buzzer_drift();	/* ÊuU[Œʒm */
	determine_vehicle_direction_and_speed();	/* ԑ̂̐isƑx肷 */
	
	/* P2{^ꂽA~ */
	if(read_sw_state(SW_P2) == SW_STATE_PUSHED){
		vehicle_control_main_line_tracing_init();
		next_lintraceing_mode_state = LINETRACING_MODE_WAITING_START;
		motor_set_speed(MOTOR_LEFT,  0 );  /* [^[͒~ */
		motor_set_speed(MOTOR_RIGHT, 0 );
	}
	/* RUNNINGLED_ */
	led_indicator_running_blink_start();
	return;	
}

/* ԑ̂̐isƑx肷 */
static void determine_vehicle_direction_and_speed(void)
{
	SINT16 linetrace_throttle_L, linetrace_throttle_R;
	
	linetrace_throttle_L = throttle_max - ((throttle_max / 10) * drift_left*3);
	linetrace_throttle_R = throttle_max - ((throttle_max / 10) * drift_right*3);
	
	
	
	
	motor_set_speed(MOTOR_LEFT,  linetrace_throttle_L );
	motor_set_speed(MOTOR_RIGHT, linetrace_throttle_R );
	
	return;
}


/* Cɑ΂鍶Ê肷 */
static void detect_drift(void)
{
	e_reclect_sensor_value sensor_L0, sensor_L1, sensor_L2;
	e_reclect_sensor_value sensor_R0, sensor_R1, sensor_R2;
	static const UINT8 drift[] = {0	,1	,3	,4	,10	,5	,8	,6};
	
	BYTE sensor_L210 = 0;
	BYTE sensor_R210 = 0;

	
	sensor_L0 = reflect_sensor_get_value(RS_OUT_L0);
	sensor_L1 = reflect_sensor_get_value(RS_OUT_L1);
	sensor_L2 = reflect_sensor_get_value(RS_OUT_L2);

	sensor_R0 = reflect_sensor_get_value(RS_OUT_R0);
	sensor_R1 = reflect_sensor_get_value(RS_OUT_R1);
	sensor_R2 = reflect_sensor_get_value(RS_OUT_R2);

	
	sensor_L210 = 0;
	sensor_L210 |= (sensor_L0 == RS_LINE_ON) ? 0x01 << 0 : 0;
	sensor_L210 |= (sensor_L1 == RS_LINE_ON) ? 0x01 << 1 : 0;
	sensor_L210 |= (sensor_L2 == RS_LINE_ON) ? 0x01 << 2 : 0;
	
	drift_left = drift[sensor_L210 & 0x07];
	
	sensor_R210= 0;
	sensor_R210 |= (sensor_R0 == RS_LINE_ON) ? 0x01 << 0 : 0;
	sensor_R210 |= (sensor_R1 == RS_LINE_ON) ? 0x01 << 1 : 0;
	sensor_R210 |= (sensor_R2 == RS_LINE_ON) ? 0x01 << 2 : 0;
	
	if( sensor_L0 == RS_LINE_ON ){
		led_indicator_turn_on(LED_IND4);
	} else {
		led_indicator_turn_off(LED_IND4);
	}
	
	if( sensor_L1 == RS_LINE_ON ){
		led_indicator_turn_on(LED_IND3);
	} else {
		led_indicator_turn_off(LED_IND3);
		
	}

	if( sensor_L2 == RS_LINE_ON ){
		led_indicator_turn_on(LED_IND2);
	} else {
		led_indicator_turn_off(LED_IND2);
		
	}
	
	
	if( sensor_R0 == RS_LINE_ON ){
		led_indicator_turn_on(LED_IND5);
	} else {
		led_indicator_turn_off(LED_IND5);
	}
	
	if( sensor_R1 == RS_LINE_ON ){
		led_indicator_turn_on(LED_IND6);
	} else {
		led_indicator_turn_off(LED_IND6);
		
	}

	if( sensor_R2 == RS_LINE_ON ){
		led_indicator_turn_on(LED_IND7);
	} else {
		led_indicator_turn_off(LED_IND7);
		
	}

	
	
	
	drift_right = drift[sensor_R210 & 0x07];
	
	return;
}

/* ÊuU[Œʒm */
static buzzer_drift(void)
{
	static SINT16 buzzer_interval_left = BUZZER_INTERVAL_INIT / 2;
	static SINT16 buzzer_interval_right = BUZZER_INTERVAL_INIT;
	static SINT16 buzzer_interval_left_init = BUZZER_INTERVAL_INIT;
	static SINT16 buzzer_interval_right_init = BUZZER_INTERVAL_INIT;
	

	
	
	buzzer_interval_left_init = BUZZER_INTERVAL_INIT / (drift_left + 1);
	buzzer_interval_right_init = BUZZER_INTERVAL_INIT / (drift_right + 1);
	

	
	buzzer_interval_left --;
	buzzer_interval_right --;
	
	if( buzzer_interval_left <= 0){
		buzzer_on_short_L();
		buzzer_interval_left = buzzer_interval_left_init;
	}
	
	if( buzzer_interval_right <= 0){
		buzzer_on_short_H();
		buzzer_interval_right = buzzer_interval_right_init;
	}
	
	return;
}