//vehicle_control_main

#include "common.h"
#include "read_sw.h"
#include "adc.h"
#include "pulse_counter.h"
#include "ring_buffer.h"
#include "serial.h"
#include "lcd.h"
#include "adc.h"
#include "motor.h"
#include "console.h"
#include "led_indicator.h"
#include "irq.h"
#include "pulse_counter.h"

#include "distance_sensor.h"
#include "remote_control.h"
#include "vehicle_control_emergyency_break.h"
#include "vehicle_control_main.h"

#define THLOTTLE_OFF_COUNT_TO_RELEASE_EMERGYENCY_BREAK (20)

static void led_indicator_remote(void);

static void vehicle_control_main_emergyency_breaking(void);
static void buzzer_remote(void);

static t_remote_control_value remote_control_value;
static e_emergyency_break_state emergyency_break_state;
static SINT16 slottle_off_count;

static BOOL break_on;

void vehicle_control_main_remote_init(void)
{
	vehicle_control_emergyency_break_init();
	break_on = FALSE;
	slottle_off_count = 0;
	return;
}

void vehicle_control_main_remote_slot_a(void)
{
	remote_control_check_command();
	remote_control_value = remote_control_get_current_value();

	vehicle_control_main_emergyency_breaking();
	
	
	return;
}


void vehicle_control_main_remote_slot_b(void)
{
	/* ً}u[L쓮ƁA܂ŌނłȂ~ */
	if( break_on == TRUE ){
		if(remote_control_value.throttle_L >= 0){
			motor_set_speed(MOTOR_LEFT,  0 );
			
		} else {
			motor_set_speed(MOTOR_LEFT,  remote_control_value.throttle_L );	
		}
		if(remote_control_value.throttle_R >= 0){
			motor_set_speed(MOTOR_RIGHT,  0 );
			
		} else {
			motor_set_speed(MOTOR_RIGHT, remote_control_value.throttle_R );	
		}

	} else { 		
		motor_set_speed(MOTOR_LEFT,  remote_control_value.throttle_L );
		motor_set_speed(MOTOR_RIGHT, remote_control_value.throttle_R );	
		
	}

	
	return;
}

void vehicle_control_main_remote_slot_c(void)
{
	led_indicator_remote();
	buzzer_remote();

	return;
}


static void vehicle_control_main_emergyency_breaking(void)
{
	static SINT16 buzzer_cycle_count;
	SINT16 buzzer_cycle_count_init = 0;
	
	static e_emergyency_break_state emergyency_break_state_before = EMERGYENCY_BREAK_NO_BARRIER_OBJECT;
	emergyency_break_state = control_emergyency_break_get_state();
	vehicle_control_emergyency_break_update_state();
	
	/* ً}u[L */
	if( emergyency_break_state == EMERGYENCY_BREAK_BARRIER_OBJECT_FRONT){
		break_on = TRUE; /* ً}u[L쓮 */
		slottle_off_count = THLOTTLE_OFF_COUNT_TO_RELEASE_EMERGYENCY_BREAK;
	}
	
	/* XbgOFFȂAXbgOFFJEg炷 */
	if( ((remote_control_value.throttle_L <= 10) && (remote_control_value.throttle_L >= -10) )
		&& 
		( (remote_control_value.throttle_R <= 10) && (remote_control_value.throttle_R >= -10) ) ){
		if(slottle_off_count > 0){
			slottle_off_count --;
		}
	}
	
	/* ً}u[L̉΁A */
	if( (break_on == TRUE) && ( slottle_off_count <= 0 ) ){
		break_on = FALSE;
		
	}
	
	/* ڋ߂ɂāAuU[̊ԊuZĂ */
	switch(emergyency_break_state){
	case	EMERGYENCY_BREAK_NO_BARRIER_OBJECT:
		buzzer_cycle_count_init = 100;				
		break;
	case EMERGYENCY_BREAK_BARRIER_OBJECT_FAR:
		buzzer_cycle_count_init = 75;				
		break;
	case EMERGYENCY_BREAK_BARRIER_OBJECT_NEAR:
		buzzer_cycle_count_init = 25;				
		break;
	case EMERGYENCY_BREAK_BARRIER_OBJECT_CLOSE:
		buzzer_cycle_count_init = 10;				
		break;
	case EMERGYENCY_BREAK_BARRIER_OBJECT_FRONT:
		buzzer_cycle_count_init = 1;				
		break;
		
			
	default  :
		buzzer_cycle_count_init = 1000;
		break;
	}
	
//	if( emergyency_break_state_before != emergyency_break_state  ){
//		buzzer_on_short_L();
//	}
	

	//QƂ̊ԊuɉăuU[炷
	if( (buzzer_cycle_count <= 0) || (emergyency_break_state_before != emergyency_break_state)){
		
		if( emergyency_break_state == EMERGYENCY_BREAK_BARRIER_OBJECT_FRONT) {
			
			//uU[́AgOXCb`T1̈ʒuɉāA炷炳Ȃ߂
			if(read_sw_state(SW_T1) == SW_STATE_DOWN){
				buzzer_on_short_H();
			}
		} else {
			//uU[́AgOXCb`T1̈ʒuɉāA炷炳Ȃ߂
			if(read_sw_state(SW_T1) == SW_STATE_DOWN){
				buzzer_on_short_L();
			}		
		}
		
		buzzer_cycle_count = buzzer_cycle_count_init;
	} else {
		buzzer_cycle_count --;
		
	}
	
	
	emergyency_break_state_before = emergyency_break_state;
	return;	
}



static void led_indicator_remote(void)
{

	if(remote_control_value.sw_toggle0 == 1){
		led_indicator_turn_on(LED_IND0);
		led_indicator_turn_on(LED_IND1);
	} else {
		led_indicator_turn_off(LED_IND0);
		led_indicator_turn_off(LED_IND1);
		
	}
	
	if(remote_control_value.sw_toggle1 == 1){
		led_indicator_turn_on(LED_IND2);		
		led_indicator_turn_on(LED_IND3);		
		led_indicator_turn_on(LED_IND4);		
		led_indicator_turn_on(LED_IND5);		
		led_indicator_turn_on(LED_IND6);		
		led_indicator_turn_on(LED_IND7);
		//putchar_to_terminal('A');
	} else {
		led_indicator_turn_off(LED_IND2);		
		led_indicator_turn_off(LED_IND3);		
		led_indicator_turn_off(LED_IND4);		
		led_indicator_turn_off(LED_IND5);		
		led_indicator_turn_off(LED_IND6);		
		led_indicator_turn_off(LED_IND7);		
		//putchar_to_terminal('B');
		
	}

	return;
}


static void buzzer_remote(void)
{

	if(remote_control_value.sw_push0 == 1){
		//buzzer_on_L();
		buzzer_on_short_L();
	} 
	if(remote_control_value.sw_push1 == 1){
		buzzer_on_short_H();
	} 
		
	if( (remote_control_value.sw_push0 == 0) && (remote_control_value.sw_push1 == 0) ) {
		//buzzer_off();		
	}

	return;
}

