#include "common.h"
#include "irq.h"
#include "gpio.h"
#include "timer.h"
#include "distance_sensor.h"

#define DISTANCE_SENSOR_UPDATE_CYCLE (5)
#define DISTANCE_SENSOR_FILTER_LENGTH (10)

static UINT16 distance_mm_filtered;

static UINT16 distance_sensor_filter[DISTANCE_SENSOR_FILTER_LENGTH];

static UINT16 distance_sensor_read_filter(void);
static void distance_sensor_write_filter(UINT16 data);


void distance_sensor_init(void)
{
	SINT16 filter_pos;
	
	for( filter_pos = 0; filter_pos < DISTANCE_SENSOR_FILTER_LENGTH; filter_pos ++){
		distance_sensor_filter[filter_pos] = 5555;
	}

	
	distance_mm_filtered = 0;
	return;
}

//ړσtB^ɐVlǉ
static void distance_sensor_write_filter(UINT16 data)
{
	static SINT16 filter_pos = 0;
	distance_sensor_filter[filter_pos] = data;
	
	filter_pos ++;
	if( filter_pos >= DISTANCE_SENSOR_FILTER_LENGTH){
		filter_pos = 0;
	}
	return;
}

//ړσtB^l擾
static UINT16 distance_sensor_read_filter(void)
{
	SINT16 filter_pos;
	UINT16 data = 0;
	
	for( filter_pos = 0; filter_pos < DISTANCE_SENSOR_FILTER_LENGTH; filter_pos ++){
		data += distance_sensor_filter[filter_pos];
	}

	
	return(data/DISTANCE_SENSOR_FILTER_LENGTH);
}

void distance_sensor_start_measure(void)
{
	gpio_set_raw_port_level(PORT_DISTANCE_SENSOR_TRIG, GPIO_HIGH);
	timer_wait_10us(2);
	gpio_set_raw_port_level(PORT_DISTANCE_SENSOR_TRIG, GPIO_LOW);
	
	
	return;
	
}

UINT16 distance_sensor_get_distance(void)
{
	return(distance_mm_filtered);
}

void distance_sensor_update_distance(void)
{
	static UINT16 count = 0;
	UINT16 timer_diff;
	UINT32 timer_diff_us;
	UINT16 distance_mm_no_filtered;
	
	
	count ++;
	
	if(count >= DISTANCE_SENSOR_UPDATE_CYCLE - 1){
		distance_sensor_start_measure();
		count = 0;
	} else {
		timer_diff = irq_get_diff();
		timer_diff_us = (timer_diff * 4) / 10; /* }CNbP */
		distance_mm_no_filtered = ((UINT32)timer_diff_us * 34) / 100;
		
		//ړσtB^ɒlǉ
		distance_sensor_write_filter(distance_mm_no_filtered);
		
		//ړςl擾
		distance_mm_filtered = distance_sensor_read_filter();
	}
	
	return;
}