#include "common.h"
#include "string.h"
#include "lcd.h"

#define LCD_COMMAND_INST (0x00)
#define LCD_COMMAND_CHAR (0x40)

#define LCD_LINE (2)
#define LCD_MSG_LENGTH (24)



static const BYTE command_initialize_lcd[] = {0x38, 0x39, 0x14, 0x70, 0x56, 0x6c};
static const BYTE command_hide_cursor[] = {LCD_COMMAND_INST, 0x0c};
static const BYTE command_show_cursor[] = {LCD_COMMAND_INST, 0x0d};
static const BYTE command_clear_lcd_show_cursor[] = {LCD_COMMAND_INST, 0x38, 0x0d, 0x01};

static const BYTE command_cursor_line_1[] = {LCD_COMMAND_INST, 0x80};
static const BYTE command_cursor_line_2[] = {LCD_COMMAND_INST, 0xc0};

static char lcd_message[LCD_LINE][LCD_MSG_LENGTH];

static BOOL lcd_need_update;

#define LCD_SLAVE_ADDR   (0x3e)

#define LCD_COMMAND_SHOW_CURSOR  (0x0d)
#define LCD_COMMAND_HIDE_CURSOR  (0x0c)

#define LCD_CHAR_MAX   (16)


void lcd_send_cmd(BYTE cmd);

static void wait(void)
{
	volatile static UINT16 count  ;
	for(count = 0; count < 100; count ++){
		/*no process*/	
	}
	
}


void lcd_init(void){
	UINT16 count;
	
	//for(count = 0; count < (sizeof(command_initialize_lcd)/sizeof(command_initialize_lcd[0])); count ++){
	//	i2c_master_send(LCD_SLAVE_ADDR,LCD_COMMAND_INST,  command_initialize_lcd[count]);
	//}
	//i2c_master_send(LCD_SLAVE_ADDR, command_initialize_lcd, sizeof(command_initialize_lcd)/sizeof(command_initialize_lcd[0]) );
	//i2c_master_send(LCD_SLAVE_ADDR, command_clear_lcd_show_cursor, sizeof(command_clear_lcd_show_cursor)/sizeof(command_clear_lcd_show_cursor[0]) );

	for(count = 0; count < sizeof(command_initialize_lcd)/sizeof(command_initialize_lcd[0]); count ++){
		lcd_send_cmd(command_initialize_lcd[count]);
	}
	for(count = 0; count < sizeof(command_clear_lcd_show_cursor)/sizeof(command_clear_lcd_show_cursor[0]); count ++){
		lcd_send_cmd(command_clear_lcd_show_cursor[count]);
	}
	
	lcd_need_update = FALSE;
	return;
}

void lcd_set_message(UINT8 line, char *msg)
{
	strncpy(lcd_message[line], msg, LCD_MSG_LENGTH - 1);
	lcd_need_update = TRUE;
	return;	
}

void lcd_control(void)
{
	if(lcd_need_update == TRUE){
		lcd_update_screen();
		lcd_need_update = FALSE;
	}
	return;	
}


/* LCD̕\XV */
void lcd_update_screen(void)
{
	UINT8 count;
	char moji;
	
	lcd_clear();  /*LCDNA*/
	lcd_cursor_to_line_1();

	wait(); /* Ȃŏ1oĂȂ̂ŁAEFCg */
	for(count = 0; count < LCD_MSG_LENGTH; count++){
		moji = lcd_message[0][count];
		if(moji == '\0'){
			break;	
		}
		lcd_putchar( moji );
		//lcd_print(lcd_message[0]);
		
	}
	
	lcd_cursor_to_line_2();
	
	for(count = 0; count < LCD_MSG_LENGTH; count++){
		moji = lcd_message[1][count];
		if(moji == '\0'){
			break;	
		}
		lcd_putchar( moji );
	}
	
	 
	return;
}

void lcd_send_cmd(BYTE cmd)
{
	BYTE data[2];
	data[0] = LCD_COMMAND_INST;
	data[1] = cmd;
	i2c_master_send(LCD_SLAVE_ADDR, data, 2);
}

void lcd_putchar(char moji)
{
	BYTE data[2];
	data[0] = LCD_COMMAND_CHAR;
	data[1] = moji;
	i2c_master_send(LCD_SLAVE_ADDR, data, 2);

	return;
}

void lcd_set_cursor_pos(unsigned int line, unsigned int left){
	
	
	return;
}

void lcd_clear(void)
{
	i2c_master_send(LCD_SLAVE_ADDR,  command_clear_lcd_show_cursor, (sizeof(command_clear_lcd_show_cursor)/sizeof(command_clear_lcd_show_cursor[0])) );
	
	return;
}

void lcd_cursor_to_line_1(void){
	i2c_master_send(LCD_SLAVE_ADDR,  command_cursor_line_1, (sizeof(command_cursor_line_1)/sizeof(command_cursor_line_1[0])) );
	
	return;
}

void lcd_cursor_to_line_2(void){
	i2c_master_send(LCD_SLAVE_ADDR,  command_cursor_line_2, (sizeof(command_cursor_line_2)/sizeof(command_cursor_line_2[0])) );
	
	return;
}

void lcd_show_cursor(void){
	i2c_master_send(LCD_SLAVE_ADDR,  command_show_cursor, (sizeof(command_show_cursor)/sizeof(command_show_cursor[0])) );
	
	return;
}

void lcd_hide_cursor(void){
	i2c_master_send(LCD_SLAVE_ADDR,  command_hide_cursor, (sizeof(command_hide_cursor)/sizeof(command_hide_cursor[0])) );

	return;
}

void lcd_print(char *message){
	char cmd[LCD_CHAR_MAX + 1] = {LCD_COMMAND_CHAR, 0x00};
	unsigned int count;

	for(count = 0; count < LCD_CHAR_MAX; count ++){
		if(message[count] == '\0'){
			break;	
		}
		cmd[count + 1] = message[count];
	}
	
	i2c_master_send(LCD_SLAVE_ADDR,  cmd, count );
	
	return;
}

