#include "common.h"
#include "pwm.h"
#include "pulse_counter.h"

#include "motor.h"

#define FILTER_COUNT (5)	/* ω */

SINT16 filter_left[FILTER_COUNT];
SINT16 filter_right[FILTER_COUNT];

UINT16 filter_pos_left;
UINT16 filter_pos_right;


#define PWM_ONE_CYCLE_COUNT (20000)

static UINT16 *duty_table;

//DUTYɑΉRyA}b`le[uŎ
//DUTY100%ɑΉl20,000
//e[u̓GNZȂǂŐ
static const UINT16 duty_table_tsukasa[] ={
0 	,
750 	,1500 	,2250 	,3000 	,3750 	,4500 	,5250 	,6000 	,
6750 	,7500 	,7950 	,8400 	,8850 	,9300 	,9750 	,10200 	,
10650 	,11100 	,11550 	,12000 	,12133 	,12267 	,12400 	,12533 	,
12667 	,12800 	,12933 	,13067 	,13200 	,13333 	,13467 	,13600 	,
13733 	,13867 	,14000 	,14133 	,14267 	,14400 	,14533 	,14667 	,
14800 	,14933 	,15067 	,15200 	,15333 	,15467 	,15600 	,15733 	,
15867 	,16000 	,16080 	,16160 	,16240 	,16320 	,16400 	,16480 	,
16560 	,16640 	,16720 	,16800 	,16880 	,16960 	,17040 	,17120 	,
17200 	,17280 	,17360 	,17440 	,17520 	,17600 	,17680 	,17760 	,
17840 	,17920 	,18000 	,18080 	,18160 	,18240 	,18320 	,18400 	,
18480 	,18560 	,18640 	,18720 	,18800 	,18880 	,18960 	,19040 	,
19120 	,19200 	,19280 	,19360 	,19440 	,19520 	,19600 	,19680 	,
19760 	,19840 	,19920 	,20000 	
};

static const UINT16 duty_table_pololu[] ={
0 	,600 	,1200 	,1800 	,2400 	,3000 	,3600 	,4200 	,4800 	,5400 	,6000 	,
6050 	,6100 	,6150 	,6200 	,6250 	,6300 	,6350 	,6400 	,6450 	,6500 	,6517 	,
6533 	,6550 	,6567 	,6583 	,6600 	,6617 	,6633 	,6650 	,6667 	,6683 	,6700 	,
6717 	,6733 	,6750 	,6767 	,6783 	,6800 	,6817 	,6833 	,6850 	,6867 	,6883 	,
6900 	,6917 	,6933 	,6950 	,6967 	,6983 	,7000 	,7100 	,
7200 	,7300 	,7400 	,7500 	,7600 	,7700 	,7800 	,7900 	,
8000 	,8100 	,8200 	,8300 	,8400 	,8500 	,8600 	,8700 	,
8800 	,8900 	,9000 	,9100 	,9200 	,9300 	,9400 	,9500 	,
9600 	,9700 	,9800 	,9900 	,10000 	,10100 	,10200 	,10300 	,
10400 	,10500 	,10600 	,10700 	,10800 	,10900 	,11000 	,11100 	,
11200 	,11300 	,11400 	,11500 	,11600 	,11700 	,11800 	,11900 	,
12000 	,


};


static SINT16 last_motor_speed_left, last_motor_speed_right;
static void motor_change_speed(BYTE motor, BYTE direction, UINT16 duty);


//PPCLK:20Mhz, :1, : 0.5ms, JEg: 10000
/*
MTU6: PWM_MOT_L1	MTIOC6A_PE7
MTU7: PWM_MOT_L2	MTIOC7A_PA2
MTU3: PWM_MOT_R1	MTIOC3A_P14
MTU4: PWM_MOT_R2	MTIOC4A_PB3
MTU8: BUZZER0		MTIOC8A_PA2
*/

void motor_init(void)
{
	
	UINT16 count;
	
	for(count = 0; count < FILTER_COUNT; count ++){
		filter_left[count] = 0;
		filter_right[count] = 0;
	}
	
	filter_pos_left = 0;
	filter_pos_right = 0;
	
	last_motor_speed_left = 0;
	last_motor_speed_right = 0;
	return;
	
}



void set_motor_type(e_motor_type motor)
{
	switch(motor){
	case MOTOR_TYPE_TSUKASA:
		duty_table = duty_table_tsukasa;	
		break;
	case MOTOR_TYPE_POLOLU:
		duty_table = duty_table_pololu;
		break;
	default:
		/* DO_NOTHING */
		break;
		
	}
	return;
}

void motor_set_speed(BYTE motor, SINT16 speed)
{
	if(speed >= 0){
		motor_change_speed(motor, ROT_CW, speed);
	} else {
		motor_change_speed(motor, ROT_CCW, speed * -1);		
	}
	
	if(motor == MOTOR_LEFT){
		last_motor_speed_left = speed;
	}
	
	if(motor == MOTOR_RIGHT){
		last_motor_speed_right = speed;
	}
	return;
}

SINT16 motor_get_speed(BYTE motor)
{
	SINT16 speed;
	if(motor == MOTOR_LEFT){
		speed = last_motor_speed_left;
	}
	
	if(motor == MOTOR_RIGHT){
		speed =last_motor_speed_right;
	}
	
	return speed;
	
}


void motor_update_speed(void)
{
	SINT16 count_left, count_right;
	
	last_motor_speed_left = count_left  = pulse_counter_read_raw_count(PULSE_CHANNEL_LEFT);
	last_motor_speed_right = count_right = pulse_counter_read_raw_count(PULSE_CHANNEL_RIGHT);
	
	filter_left[filter_pos_left] = count_left;
	filter_right[filter_pos_right] = count_right;
	
	
	filter_pos_left ++;
	filter_pos_right ++;
	
	if(filter_pos_left >= FILTER_COUNT) {
		filter_pos_left = 0;
	} 
	
	if(filter_pos_right >= FILTER_COUNT) {
		filter_pos_right = 0;
	} 
	
	return;
}

static void motor_change_speed(BYTE motor, BYTE direction, UINT16 duty)
{
	unsigned int value_a;
	unsigned int value_b;
	
	if(duty == 0){
		value_a = PWM_ONE_CYCLE_COUNT - 1;
		value_b = value_a;
	} else if(duty >= 100){
		value_a = PWM_ONE_CYCLE_COUNT - 1;
		//value_b = value_a + 1;
		value_b = duty_table[100];
	} else {
		value_a = PWM_ONE_CYCLE_COUNT - 1;
		//value_b = (((UINT32)PWM_ONE_CYCLE_COUNT * duty) / 100)  - 1;
		value_b = duty_table[duty];
	}
	
	switch(motor){
	case MOTOR_LEFT:
		if(duty > 0){
			if(direction == ROT_CW){
				pwm_set_cycle_duty_MTIOC6A(value_a, value_b); //PWM_MOT_L1 -> IN1 -> PWM
				pwm_set_off_MTIOC7A(); //PWM_MOT_L2 -> IN2 -> L(DUTY:0%)
				
			} else if(direction == ROT_CCW){
				pwm_set_off_MTIOC6A(); //PWM_MOT_L1 -> IN1 -> L(DUTY:0%)
				pwm_set_cycle_duty_MTIOC7A(value_a, value_b); //PWM_MOT_L2 -> IN2 -> PWM
				
			} else {
				pwm_set_off_MTIOC6A(); //PWM_MOT_L1 -> IN1 -> (DUTY:0%)
				pwm_set_off_MTIOC7A(); //PWM_MOT_L2 -> IN2 -> (DUTY:0%)
			}
		} else {
			pwm_set_off_MTIOC7A(); //PWM_MOT_L1 -> IN1 -> (DUTY:0%)
			pwm_set_off_MTIOC6A(); //PWM_MOT_L2 -> IN2 -> (DUTY:0%)
		}
		break;
	case MOTOR_RIGHT:
		if(duty > 0){
			if(direction == ROT_CW){
				pwm_set_cycle_duty_MTIOC4A(value_a, value_b); //PWM_MOT_R2 -> IN2 -> PWM
				pwm_set_off_MTIOC3A(); //PWM_MOT_R1 -> IN1 -> L(DUTY:0%)
                
			} else if(direction == ROT_CCW){
				pwm_set_off_MTIOC4A(); //PWM_MOT_R2 -> IN2 -> L(DUTY:0%)
				pwm_set_cycle_duty_MTIOC3A(value_a, value_b); //PWM_MOT_R1 -> IN1 -> PWM
				
			} else {
				pwm_set_off_MTIOC3A();   //PWM_MOT_R1 -> IN1 -> (DUTY:0%)
				pwm_set_off_MTIOC4A();   //PWM_MOT_R2 -> IN2 -> (DUTY:0%)	
			}
		} else {
			pwm_set_off_MTIOC4A();   //PWM_MOT_R2 -> IN2 -> (DUTY:0%)				
			pwm_set_off_MTIOC3A();   //PWM_MOT_R1 -> IN1 -> (DUTY:0%)
		}
		break;
	default:
	
		break;
	}
	
	return;
}

