//read_send_udp_lcd.ino

//include
#include <WiFi.h>     //wifi関連
#include <WiFiUdp.h> 

#include<Wire.h>  //I2C通信ライブラリ
#include<stdio.h>  

#define LCD_I2C_ADDRESS (0x3E) //I2Cアドレス

//●送信先(模型車両) RaspberryPiのIPアドレス
static const char *kRemoteIpadr = "192.168.100.123";  //送信先のIPアドレス
static const int kRmoteUdpPort = 12345;           //送信先のポート番号

//●WIFIアクセスポイント
const char ssid[] = "xxxxxxxxxx";    // wifi-AP SSID
const char password[] = "xxxxxxxxxx";      // wifi-AP password

#define WIFI_STATIC  //DHCPを使う場合はコメントアウトする

#ifdef WIFI_STATIC
//●送信機(自機)のIPアドレス設定
//※オクテットの区切りが . (ドット) でなく ,(カンマ)であることに注意
const IPAddress ip(192, 168, 100, 45);
const IPAddress gateway(192, 168, 100, 254);
const IPAddress subnet(255, 255, 255, 0);
const IPAddress dns1(192, 168, 100, 254);
#endif

static const int kLocalPort = 5000;  //自機のポート番号

static WiFiUDP wifiUdp;
bool wifi_established;

//シリアル関連速度
#define SERIAL_BAUD_RATE (38400)

//スイッチ関連の番号
#define SW_PUSH0 (13)
#define SW_PUSH1 (14)
#define SW_TOGGLE0 (26)
#define SW_TOGGLE1 (27)

//LED関連の番号
#define LED_IND0  (16)
#define LED_IND1  (17)
#define LED_IND2  (18)
#define LED_IND3  (19)

//可変抵抗(スロットル)の番号
#define  VR_THROTTLE_L  (34)
#define  VR_THROTTLE_R  (35)

//1サイクルあたりのウェイト時間
#define DELAY_ONE_CYCLE_MS  (33)

#define REMOTE_CORRECT_CHECK (9876)  /* 正常受信できたかチェックするための合い言葉 */

//wifiステータスチェックを一旦諦めるカウント数
#define WIFI_RETRY_COUNT (100)

//次のwifiステータスチェックまでの待ち時間
#define WIFI_STATUS_CHECK_CYCLE_MS (100)

//操作パネルの値をまとめる構造体
typedef struct {
  int sw_push0;
  int sw_push1;
  int sw_toggle0;
  int sw_toggle1;
  int throttle_left;
  int throttle_right;
} t_transmitter_sw_ad;


//初期化処理
void setup() {
  // put your setup code here, to run once:
  delay(100);
  init_sw_led();
  //赤LED点灯
  digitalWrite(LED_IND3,  LOW); //LED_IND3(赤) 点灯
  digitalWrite(LED_IND0,  LOW); //LED_IND0 点灯 
  digitalWrite(LED_IND1,  HIGH); //LED_IND1 消灯 
  digitalWrite(LED_IND2,  HIGH); //LED_IND2 消灯 

  Wire.begin();
  lcd_init();  //lcd初期化

  Serial.begin(SERIAL_BAUD_RATE);

  //LCDに初期化中を表すメッセージを表示する
  lcd_set_cursor_line_1();
  lcd_write_str("CONNECTING WIFI...");
  lcd_clear();
  lcd_write_str("TRANSMITTER BOARD");
  lcd_set_cursor_line_2();
  lcd_write_str("IS STARTED!!");

  delay(1000);

  //wifiに接続する
  while(1){
    //LCDパネルにwifi接続中を示すメッセージを表示する
    lcd_clear();
    lcd_set_cursor_line_1();
    lcd_write_str("CONNECTING WIFI...");

    
    digitalWrite(LED_IND3, LOW);   //赤LED点灯
    digitalWrite(LED_IND2, HIGH);

    //wifi接続する
    if( init_wifi() == true){

      //wifi接続できた
      digitalWrite(LED_IND2, LOW);  //LED_IND2 点灯
      digitalWrite(LED_IND1, HIGH);//LED_IND1 消灯

      lcd_set_cursor_line_1();
      lcd_write_str("WIFI CONNECTED!");

      break; //無限ループを抜ける
    } else {
      //wifi接続できなかったので。もういちど接続を試す
      digitalWrite(LED_IND3,  LOW); //赤LED点灯
      delay(100); //すこしだけウェイトをいれる
      //LCDエラーメッセージ表示
      lcd_clear();
      lcd_set_cursor_line_1();
      lcd_write_str("WIFI FAILED.");
      lcd_set_cursor_line_2();
      lcd_write_str("STARTING...");
      delay(2000);  //すこしだけウェイトをいれる
    }
  }
}


//メイン処理
void loop() {
  t_transmitter_sw_ad sw_ad;
  int adval_l, adval_r;
  char send_buf[1024];
  char lcd_buf_line1[16];
  char lcd_buf_line2[16];
  static int lcd_cycle = 0;
  static unsigned long int time_count = 0;

  // put your main code here, to run repeatedly:
  adval_r = analogRead(35);
  adval_l = analogRead(34);

  digitalWrite(LED_IND0, LOW);   //LED_IND0(D1)は点灯
  digitalWrite(LED_IND3, HIGH);  //LED_IND3(D4)は消灯

  sw_ad =update_sw_ad();

  sprintf(send_buf, "REMOTE %4d, %4d, %d, %d, %d, %d, %d", 

    sw_ad.throttle_left, sw_ad.throttle_right, sw_ad.sw_push0, sw_ad.sw_push1
    ,sw_ad.sw_toggle0,sw_ad.sw_toggle1,
        REMOTE_CORRECT_CHECK);

    //シリアル送信
    Serial.println(send_buf);

    //wifi送信
   if ( wifi_established == true ) {
    wifiUdp.beginPacket(kRemoteIpadr, kRmoteUdpPort);
    wifiUdp.print(send_buf); 
    wifiUdp.endPacket();
   }
    //LED_IND1を、送信タイミングにあわせて点滅させる
    if( time_count % 2 == 0 ){
    digitalWrite(LED_IND1, HIGH);
   } else {
    digitalWrite(LED_IND1, LOW);
  }

  time_count ++;

  Serial.println(send_buf);

  lcd_cycle ++;
  if(lcd_cycle % 2  == 0){
    sprintf(lcd_buf_line1, "L:%4d%%, R:%4d%%", sw_ad.throttle_left, sw_ad.throttle_right);
    sprintf(lcd_buf_line2, "SW: %1d %1d %1d %1d     ", sw_ad.sw_push0, sw_ad.sw_toggle0,  sw_ad.sw_toggle1, sw_ad.sw_push1);
    
    //lcd_clear();
    lcd_set_cursor_line_1();
    lcd_write_str(lcd_buf_line1);

  } else {
    lcd_set_cursor_line_2();
    lcd_write_str(lcd_buf_line2);

  }
  delay(DELAY_ONE_CYCLE_MS);
}

//LED関連の初期設定
void init_sw_led(void){
    //initialilze sw
  pinMode(SW_PUSH0, INPUT);
  pinMode(SW_PUSH1, INPUT);
  pinMode(SW_TOGGLE0, INPUT);
  pinMode(SW_TOGGLE1, INPUT);

  //initialze led
  pinMode(LED_IND0, OUTPUT);
  pinMode(LED_IND1, OUTPUT);
  pinMode(LED_IND2, OUTPUT);
  pinMode(LED_IND3, OUTPUT);

  digitalWrite(LED_IND0, HIGH);
  digitalWrite(LED_IND1, HIGH);
  digitalWrite(LED_IND2, HIGH);
  digitalWrite(LED_IND3, HIGH);

}

//wifiに接続する
bool init_wifi(void){
  int retry_count = 0;
  wifi_established = true;
  if (!WiFi.config(ip,gateway,subnet,dns1)){
      Serial.println("Failed to configure!");
      wifi_established = false;
      return wifi_established;
  }
  
  //wifi接続を開始する
  WiFi.begin(ssid, password);
  while ( WiFi.status() != WL_CONNECTED) {
    retry_count ++;
    if ( retry_count > WIFI_RETRY_COUNT ) {
      wifi_established = false;
      break;
    }
    delay(WIFI_STATUS_CHECK_CYCLE_MS);
    //接続操作中に、赤LEDを点滅させて、頑張ってる感をだす
    if( (retry_count % 2) == 0){
      digitalWrite(LED_IND3, HIGH);
    } else {
      digitalWrite(LED_IND3, LOW);
    }
  }
  if ( wifi_established == true ) wifiUdp.begin(kLocalPort);

  return(wifi_established);
}

//LCD関連の初期化
void lcd_init(void){
  lcd_write_command( 0x38);
  lcd_write_command( 0x39);
  lcd_write_command( 0x14);
  lcd_write_command( 0x70);
  lcd_write_command( 0x56);
  lcd_write_command( 0x6c);
  lcd_write_command( 0x0d);
  lcd_write_command( 0x01);

  delay(100);

  lcd_write_str("INITIALIZING");

}

void lcd_clear(void){
  lcd_write_command( 0x01);
  
}

void lcd_show_cursor(void){
  lcd_write_command(0x0d);
  
}

void lcd_hide_cursor(void){
  lcd_write_command(0x0c);
  
}

void lcd_set_cursor_line_1(void){
  lcd_write_command(0x80);
  
}

void lcd_set_cursor_line_2(void){
  lcd_write_command(0xc0);
  
}

void lcd_write_str(char *msg){
  Wire.beginTransmission(LCD_I2C_ADDRESS); 
  Wire.write(0x40);

  for(int i = 0; i < 16; i++){
    if( msg[i] == '\0') break;
    Wire.write(msg[i]);
  }

  Wire.endTransmission();
  delay(5);

}

//スロットルの値を読み込む
t_transmitter_sw_ad update_sw_ad(void) {
  t_transmitter_sw_ad sw_ad;
  int ad_left, ad_right;
  float throttle_l, throttle_r;

  sw_ad.sw_push0 = (digitalRead(SW_PUSH0) == HIGH) ?  1 : 0;
  sw_ad.sw_push1 = (digitalRead(SW_PUSH1) == HIGH) ?  1 : 0;
  sw_ad.sw_toggle0 = (digitalRead(SW_TOGGLE0) == LOW) ?  1 : 0;
  sw_ad.sw_toggle1 = (digitalRead(SW_TOGGLE1) == LOW) ?  1 : 0;

  ad_left = analogRead(VR_THROTTLE_L);
  ad_right = analogRead(VR_THROTTLE_R);

  //0-1400: -100-0, 1800-4095: 0-100
  if( (ad_left >=0) && (ad_left <= 1400) ){
      throttle_l = ( 1.0 - (float(ad_left) - 0 ) / float(1400 - 0)) * -100.0;
  } else if((ad_left >=1800) && (ad_left <= 4095)){
      throttle_l = ((float(ad_left) - 1800 ) / float(4095 - 1800)) * 100.0;
  } else {
    throttle_l = 0;
  }
  sw_ad.throttle_left = throttle_l;

  if( (ad_right >=0) && (ad_right <= 1400) ){
      throttle_r = ( 1.0 - (float(ad_right) - 0 ) / float(1400 - 0)) * -100.0;
  } else if((ad_right >=1800) && (ad_right <= 4095)){
      throttle_r = ((float(ad_right) - 1800 ) / float(4095 - 1800)) * 100.0;
  } else {
    throttle_r = 0;
  }
  sw_ad.throttle_right = throttle_r;

  return sw_ad;
}


void lcd_write_command(byte command){
  Wire.beginTransmission(LCD_I2C_ADDRESS); 
  Wire.write(0x00);
  Wire.write(command); 
  Wire.endTransmission();
  delay(5);  //10ms待つ

}

void lcd_write_char(char moji){
  Wire.beginTransmission(LCD_I2C_ADDRESS); 
  Wire.write(0x40);
  Wire.write(moji); 
  Wire.endTransmission();
  delay(5);  //10ms待つ

}
